# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock

from heatclient import exc as heat_exc
import six
from swiftclient import exceptions as swiftexceptions

from tripleo_common.tests import base
from tripleo_common.utils import overcloudrc


class OvercloudRcTest(base.TestCase):

    def test_generate_overcloudrc(self):

        stack = mock.MagicMock()
        stack.stack_name = 'overcast'
        stack.to_dict.return_value = {
            "outputs": [
                {'output_key': 'KeystoneURL',
                 'output_value': 'http://foo.com:8000/'},
                {'output_key': 'EndpointMap',
                 'output_value': {'KeystoneAdmin': {'host': 'fd00::1'}}},
            ]
        }

        result = overcloudrc._create_overcloudrc(stack, "foo", "AdminPassword",
                                                 "regionTwo")

        self.assertIn("export no_proxy='[fd00::1],foo,foo.com'",
                      result['overcloudrc'])
        self.assertIn("OS_PASSWORD=AdminPassword", result['overcloudrc'])

        self.assertIn("export PYTHONWARNINGS='ignore:Certificate",
                      result['overcloudrc'])
        self.assertIn("OS_IDENTITY_API_VERSION=3", result['overcloudrc'])
        self.assertIn(overcloudrc.CLOUDPROMPT, result['overcloudrc'])
        self.assertIn("OS_AUTH_TYPE=password", result['overcloudrc'])
        self.assertIn("OS_AUTH_URL=http://foo.com:8000/",
                      result['overcloudrc'])
        self.assertIn("OS_REGION_NAME=regionTwo",
                      result['overcloudrc'])

    def test_generate_overcloudrc_with_duplicated_no_proxy(self):

        stack = mock.MagicMock()
        stack.stack_name = 'overcast'
        stack.to_dict.return_value = {
            "outputs": [
                {'output_key': 'KeystoneURL',
                 'output_value': 'http://foo.com:8000/'},
                {'output_key': 'EndpointMap',
                 'output_value': {'KeystoneAdmin': {'host': 'fd00::1'}}},
            ]
        }

        result = overcloudrc._create_overcloudrc(
            stack, "foo,foo.com", "AdminPassword", "regionTwo")

        self.assertIn("export no_proxy='[fd00::1],foo,foo.com'",
                      result['overcloudrc'])
        self.assertIn("OS_PASSWORD=AdminPassword", result['overcloudrc'])

        self.assertIn("export PYTHONWARNINGS='ignore:Certificate",
                      result['overcloudrc'])
        self.assertIn("OS_IDENTITY_API_VERSION=3", result['overcloudrc'])
        self.assertIn(overcloudrc.CLOUDPROMPT, result['overcloudrc'])
        self.assertIn("OS_AUTH_TYPE=password", result['overcloudrc'])
        self.assertIn("OS_AUTH_URL=http://foo.com:8000/",
                      result['overcloudrc'])
        self.assertIn("OS_REGION_NAME=regionTwo",
                      result['overcloudrc'])

    def test_overcloudrc_no_stack(self):
        mock_swift = mock.MagicMock()
        mock_heat = mock.MagicMock()
        not_found = heat_exc.HTTPNotFound()
        mock_heat.stacks.get.side_effect = not_found
        ex = self.assertRaises(RuntimeError,
                               overcloudrc.create_overcloudrc,
                               mock_swift, mock_heat, "overcast")

        self.assertEqual((
            "The Heat stack overcast could not be found. Make sure you have "
            "deployed before calling this action."
        ), six.text_type(ex))

    def test_overcloudrc_no_env(self):
        mock_swift = mock.MagicMock()
        mock_heat = mock.MagicMock()
        mock_swift.get_object.side_effect = (
            swiftexceptions.ClientException("overcast"))
        ex = self.assertRaises(RuntimeError,
                               overcloudrc.create_overcloudrc,
                               mock_swift, mock_heat, "overcast")

        self.assertEqual("Error retrieving environment for plan overcast: "
                         "overcast", six.text_type(ex))

    def test_overcloudrc_no_password(self):
        mock_swift = mock.MagicMock()
        mock_heat = mock.MagicMock()
        mock_swift.get_object.return_value = (
            {}, "version: 1.0")
        ex = self.assertRaises(RuntimeError,
                               overcloudrc.create_overcloudrc,
                               mock_swift, mock_heat, "overcast")

        self.assertEqual(
            "Unable to find the AdminPassword in the plan environment.",
            six.text_type(ex))

    @mock.patch('tripleo_common.utils.overcloudrc._create_overcloudrc')
    def test_success(self, mock_create_overcloudrc):

        mock_env = """
        version: 1.0

        template: overcloud.yaml
        environments:
        - path: overcloud-resource-registry-puppet.yaml
        - path: environments/services/sahara.yaml
        parameter_defaults:
          BlockStorageCount: 42
          OvercloudControlFlavor: yummy
        passwords:
          AdminPassword: SUPERSECUREPASSWORD
        """
        mock_swift = mock.MagicMock()
        mock_heat = mock.MagicMock()
        mock_swift.get_object.return_value = ({}, mock_env)
        mock_create_overcloudrc.return_value = {
            "overcloudrc": "fake overcloudrc"
        }

        result = overcloudrc.create_overcloudrc(
            mock_swift, mock_heat, "overcast")
        self.assertEqual(result, {"overcloudrc": "fake overcloudrc"})



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import jinja2
import logging
import os
import six
import tempfile
import yaml

from heatclient import exc as heat_exc
from swiftclient import exceptions as swiftexceptions

from tripleo_common import constants
from tripleo_common.utils import parameters
from tripleo_common.utils import plan as plan_utils
from tripleo_common.utils import swift as swiftutils
from tripleo_common.utils import tarball

LOG = logging.getLogger(__name__)


class J2SwiftLoader(jinja2.BaseLoader):
    """Jinja2 loader to fetch included files from swift

    This attempts to fetch a template include file from the given container.
    An optional search path or list of search paths can be provided. By default
    only the absolute path relative to the container root is searched.
    """

    def __init__(self, swift, container, searchpath):
        self.swift = swift
        self.container = container
        self.searchpath = [searchpath]
        # Always search the absolute path from the root of the swift container
        if '' not in self.searchpath:
            self.searchpath.append('')

    def get_source(self, environment, template):
        pieces = jinja2.loaders.split_template_path(template)
        for searchpath in self.searchpath:
            template_path = os.path.join(searchpath, *pieces)
            try:
                source = swiftutils.get_object_string(self.swift,
                                                      self.container,
                                                      template_path)
                return source, None, False
            except swiftexceptions.ClientException:
                pass
        raise jinja2.exceptions.TemplateNotFound(template)


def j2_render_and_put(swift, j2_template, j2_data, yaml_f,
                      container=constants.DEFAULT_CONTAINER_NAME):

    def raise_helper(msg):
        raise jinja2.exceptions.TemplateError(msg)

    # Search for templates relative to the current template path first
    template_base = os.path.dirname(yaml_f)
    j2_loader = J2SwiftLoader(swift, container, template_base)

    try:
        # Render the j2 template
        jinja2_env = jinja2.Environment(loader=j2_loader)
        jinja2_env.globals['raise'] = raise_helper
        template = jinja2_env.from_string(j2_template)
        r_template = template.render(**j2_data)
    except jinja2.exceptions.TemplateError as ex:
        error_msg = ("Error rendering template %s : %s"
                     % (yaml_f, six.text_type(ex)))
        LOG.error(error_msg)
        raise RuntimeError(error_msg)

    try:
        # write the template back to the plan container
        LOG.info("Writing rendered template %s" % yaml_f)
        swiftutils.put_object_string(swift, container, yaml_f,
                                     r_template)
    except swiftexceptions.ClientException:
        error_msg = ("Error storing file %s in container %s"
                     % (yaml_f, container))
        LOG.error(error_msg)
        raise RuntimeError(error_msg)


def get_j2_excludes_file(swift, container=constants.DEFAULT_CONTAINER_NAME):
    try:
        j2_excl_file = swiftutils.get_object_string(
            swift, container, constants.OVERCLOUD_J2_EXCLUDES)
        j2_excl_data = yaml.safe_load(j2_excl_file)
        if (j2_excl_data is None or j2_excl_data.get('name') is None):
            j2_excl_data = {"name": []}
            LOG.info("j2_excludes.yaml is either empty or there are "
                     "no templates to exclude, defaulting the J2 "
                     "excludes list to: %s" % j2_excl_data)
    except swiftexceptions.ClientException:
        j2_excl_data = {"name": []}
        LOG.info("No J2 exclude file found, defaulting "
                 "the J2 excludes list to: %s" % j2_excl_data)
    return j2_excl_data


def heat_resource_exists(heat, stack, nested_stack_name, resource_name):
    if stack is None:
        LOG.debug("Resource does not exist because stack does not exist")
        return False

    try:
        nested_stack = heat.resources.get(stack.id, nested_stack_name)
    except heat_exc.HTTPNotFound:
        LOG.debug(
            "Resource does not exist because {} stack does "
            "not exist".format(nested_stack_name))
        return False

    try:
        heat.resources.get(nested_stack.physical_resource_id,
                           resource_name)
    except heat_exc.HTTPNotFound:
        LOG.debug("Resource does not exist: {}".format(resource_name))
        return False
    else:
        LOG.debug("Resource exists: {}".format(resource_name))
        return True


def process_custom_roles(swift, heat,
                         container=constants.DEFAULT_CONTAINER_NAME):
    try:
        j2_role_file = swiftutils.get_object_string(
            swift, container, constants.OVERCLOUD_J2_ROLES_NAME)
        role_data = yaml.safe_load(j2_role_file)
    except swiftexceptions.ClientException:
        LOG.info("No %s file found, skipping jinja templating"
                 % constants.OVERCLOUD_J2_ROLES_NAME)
        return

    try:
        j2_network_file = swiftutils.get_object_string(
            swift, container, constants.OVERCLOUD_J2_NETWORKS_NAME)
        network_data = yaml.safe_load(j2_network_file)
        # Allow no networks defined in network_data
        if network_data is None:
            network_data = []
    except swiftexceptions.ClientException:
        # Until t-h-t contains network_data.yaml we tolerate a missing file
        LOG.warning("No %s file found, ignoring"
                    % constants.OVERCLOUD_J2_ROLES_NAME)
        network_data = []

    j2_excl_data = get_j2_excludes_file(swift, container)

    try:
        # Iterate over all files in the plan container
        # we j2 render any with the .j2.yaml suffix
        container_files = swift.get_container(container)
    except swiftexceptions.ClientException as ex:
        error_msg = ("Error listing contents of container %s : %s"
                     % (container, six.text_type(ex)))
        LOG.error(error_msg)
        raise RuntimeError(error_msg)

    role_names = [r.get('name') for r in role_data]
    r_map = {}
    for r in role_data:
        r_map[r.get('name')] = r
    excl_templates = j2_excl_data.get('name')

    stack = None
    try:
        stack = heat.stacks.get(container, resolve_outputs=False)
    except heat_exc.HTTPNotFound:
        LOG.debug("Stack does not exist")

    n_map = {}
    for n in network_data:
        if n.get('enabled') is not False:
            n_map[n.get('name')] = n
            if not n.get('name_lower'):
                n_map[n.get('name')]['name_lower'] = n.get('name').lower()
        if n.get('name') == constants.API_NETWORK and 'compat_name' \
                not in n.keys():
            # Check to see if legacy named API network exists
            # and if so we need to set compat_name
            api_net = "{}Network".format(constants.LEGACY_API_NETWORK)
            if heat_resource_exists(heat, stack, 'Networks', api_net):
                n['compat_name'] = 'Internal'
                LOG.info("Upgrade compatibility enabled for legacy "
                         "network resource Internal.")
        else:
            LOG.info("skipping %s network: network is disabled." %
                     n.get('name'))

    plan_utils.cache_delete(swift, container, "tripleo.parameters.get")

    for f in [f.get('name') for f in container_files[1]]:
        # We do three templating passes here:
        # 1. *.role.j2.yaml - we template just the role name
        #    and create multiple files (one per role)
        # 2  *.network.j2.yaml - we template the network name and
        #    data and create multiple files for networks and
        #    network ports (one per network)
        # 3. *.j2.yaml - we template with all roles_data,
        #    and create one file common to all roles
        if f.endswith('.role.j2.yaml'):
            LOG.info("jinja2 rendering role template %s" % f)
            j2_template = swiftutils.get_object_string(swift,
                                                       container, f)
            LOG.info("jinja2 rendering roles %s" % ","
                     .join(role_names))
            for role in role_names:
                LOG.info("jinja2 rendering role %s" % role)
                out_f = "-".join(
                    [role.lower(),
                     os.path.basename(f).replace('.role.j2.yaml',
                                                 '.yaml')])
                out_f_path = os.path.join(os.path.dirname(f), out_f)
                if ('network/config' in os.path.dirname(f) and
                        r_map[role].get('deprecated_nic_config_name')):
                    d_name = r_map[role].get('deprecated_nic_config_name')
                    out_f_path = os.path.join(os.path.dirname(f), d_name)
                elif ('network/config' in os.path.dirname(f)):
                    d_name = "%s.yaml" % role.lower()
                    out_f_path = os.path.join(os.path.dirname(f), d_name)
                if not (out_f_path in excl_templates):
                    if '{{role.name}}' in j2_template:
                        j2_data = {'role': r_map[role],
                                   'networks': network_data}
                        j2_render_and_put(swift, j2_template,
                                          j2_data, out_f_path,
                                          container)
                    else:
                        # Backwards compatibility with templates
                        # that specify {{role}} vs {{role.name}}
                        j2_data = {'role': role, 'networks': network_data}
                        LOG.debug("role legacy path for role %s" % role)
                        j2_render_and_put(swift, j2_template,
                                          j2_data, out_f_path,
                                          container)
                else:
                    LOG.info("Skipping rendering of %s, defined in %s" %
                             (out_f_path, j2_excl_data))

        elif (f.endswith('.network.j2.yaml')):
            LOG.info("jinja2 rendering network template %s" % f)
            j2_template = swiftutils.get_object_string(swift,
                                                       container,
                                                       f)
            LOG.info("jinja2 rendering networks %s" % ",".join(n_map))
            for network in n_map:
                j2_data = {'network': n_map[network]}
                # Output file names in "<name>.yaml" format
                out_f = os.path.basename(f).replace('.network.j2.yaml',
                                                    '.yaml')
                if os.path.dirname(f).endswith('ports'):
                    out_f = out_f.replace('port',
                                          n_map[network]['name_lower'])
                else:
                    out_f = out_f.replace('network',
                                          n_map[network]['name_lower'])
                out_f_path = os.path.join(os.path.dirname(f), out_f)
                if not (out_f_path in excl_templates):
                    j2_render_and_put(swift, j2_template,
                                      j2_data, out_f_path,
                                      container)
                else:
                    LOG.info("Skipping rendering of %s, defined in %s" %
                             (out_f_path, j2_excl_data))

        elif f.endswith('.j2.yaml'):
            LOG.info("jinja2 rendering %s" % f)
            j2_template = swiftutils.get_object_string(swift,
                                                       container,
                                                       f)
            j2_data = {'roles': role_data, 'networks': network_data}
            out_f = f.replace('.j2.yaml', '.yaml')
            j2_render_and_put(swift, j2_template,
                              j2_data, out_f,
                              container)
    return role_data


def prune_unused_services(swift, role_data,
                          resource_registry,
                          container=constants.DEFAULT_CONTAINER_NAME):
    """Remove unused services from role data

    Finds the unused services in the resource registry and removes them
    from the role data in the plan so we do not create OS::Heat::None
    resources.

    :param resource_registry: tripleo resource registry dict
    :param swift: swift client
    :param resource_registry: tripleo resource registry dict
    :returns: true if we updated the roles file. else false
    """
    to_remove = set()
    for key, value in resource_registry.items():
        if (key.startswith('OS::TripleO::Services::') and
                value.startswith('OS::Heat::None')):
            to_remove.add(key)

    if not to_remove or not role_data:
        LOG.info('No unused services to prune or no role data')
        return False

    LOG.info('Removing unused services from role data')
    for role in role_data:
        role_name = role.get('name')
        for service in to_remove:
            try:
                role.get('ServicesDefault', []).remove(service)
                LOG.debug('Removing {} from {} role'.format(
                    service, role_name))
            except ValueError:
                pass
    LOG.debug('Saving updated role data to swift')
    swift.put_object(container,
                     constants.OVERCLOUD_J2_ROLES_NAME,
                     yaml.safe_dump(role_data,
                                    default_flow_style=False))
    return True


def build_heat_args(swift, heat, container=constants.DEFAULT_CONTAINER_NAME):
    error_text = None
    try:
        plan_env = plan_utils.get_env(swift, container)
    except swiftexceptions.ClientException as err:
        err_msg = ("Error retrieving environment for plan %s: %s" % (
            container, err))
        LOG.exception(err_msg)
        raise RuntimeError(error_text)

    try:
        # if the jinja overcloud template exists, process it and write it
        # back to the swift container before continuing processing.  The
        # method called below should handle the case where the files are
        # not found in swift, but if they are found and an exception
        # occurs during processing, then it will be raised.
        role_data = process_custom_roles(swift, heat, container)
    except Exception as err:
        LOG.exception("Error occurred while processing custom roles.")
        raise RuntimeError(six.text_type(err))

    template_name = plan_env.get('template', "")

    template_object = os.path.join(swift.url, container,
                                   template_name)
    LOG.debug('Template: %s' % template_name)
    try:
        template_files, template = plan_utils.get_template_contents(
            swift, template_object)
    except Exception as err:
        error_text = six.text_type(err)
        LOG.exception("Error occurred while fetching %s" % template_object)

    temp_env_paths = []
    try:
        env_paths, temp_env_paths = plan_utils.build_env_paths(
            swift, container, plan_env)
        env_files, env = plan_utils.process_environments_and_files(
            swift, env_paths)
        parameters.convert_docker_params(env)

    except Exception as err:
        error_text = six.text_type(err)
        LOG.exception("Error occurred while processing plan files.")
    finally:
        # cleanup any local temp files
        for f in temp_env_paths:
            os.remove(f)
    if error_text:
        raise RuntimeError(six.text_type(error_text))

    heat_args = {
        'template': template,
        'template_files': template_files,
        'env': env,
        'env_files': env_files
    }
    return heat_args, role_data


def process_templates(swift, heat, container=constants.DEFAULT_CONTAINER_NAME,
                      prune_services=False):
    heat_args, role_data = build_heat_args(swift, heat, container)
    if prune_services:
        try:
            # Prune OS::Heat::None resources
            resource_reg = heat_args['env'].get('resource_registry', {})
            roles_updated = prune_unused_services(
                swift, role_data, resource_reg, container)
            if roles_updated:
                heat_args, _ = build_heat_args(swift, heat, container)

        except Exception as err:
            LOG.exception("Error occurred while prunning prune_services.")
            raise RuntimeError(six.text_type(err))

    files = dict(list(heat_args['template_files'].items()) + list(
        heat_args['env_files'].items()))

    return {
        'stack_name': container,
        'template': heat_args['template'],
        'environment': heat_args['env'],
        'files': files
    }


def upload_templates_as_tarball(
    swift, dir_to_upload=constants.DEFAULT_TEMPLATES_PATH,
    container=constants.DEFAULT_CONTAINER_NAME):
    with tempfile.NamedTemporaryFile() as tmp_tarball:
        tarball.create_tarball(dir_to_upload, tmp_tarball.name)
        tarball.tarball_extract_to_swift_container(
            swift,
            tmp_tarball.name,
            container)



# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from unittest import mock

from collections import namedtuple
import yaml

from tripleo_common.constants import PLAN_NAME_PATTERN
from tripleo_common.tests import base
from tripleo_common.utils import validations


VALIDATION_DEFAULT = """---
- hosts: overcloud
  vars:
    metadata:
      name: First validation
      description: Default validation
  tasks:
  - name: Ping the nodes
    ping:
"""

VALIDATION_CUSTOM = """---
- hosts: overcloud
  vars:
    metadata:
      name: First validation
      description: Custom validation
  tasks:
  - name: Ping the nodes
    ping:
"""

VALIDATION_GROUP_1 = """---
- hosts: overcloud
  vars:
    metadata:
      name: First validation
      description: A validation belonging to group1
      groups:
        - group1
  tasks:
  - name: Ping the nodes
    ping:
"""

VALIDATION_WITH_METADATA = """---
- hosts: undercloud
  vars:
    metadata:
      name: Validation with metadata
      description: A validation with extra metadata
    foo: a foo metadata
    bar: 42
  tasks:
  - name: Do something useful
    watch_tv:
"""

VALIDATION_GROUPS_1_2 = """---
- hosts: undercloud
  vars:
    metadata:
      name: Validation from many groups
      description: A validation belonging to groups 1 and 2
      groups:
        - group1
        - group2
  tasks:
  - name: Do something useful
    watch_tv:
"""

VALIDATION_GROUP_1_PARSED = {
    'description': 'A validation belonging to group1',
    'groups': ['group1'],
    'id': 'VALIDATION_GROUP_1',
    'parameters': {},
    'name': 'First validation',
}

VALIDATION_WITH_METADATA_PARSED = {
    'description': 'A validation with extra metadata',
    'groups': [],
    'id': 'VALIDATION_WITH_METADATA',
    'parameters': {'foo': 'a foo metadata', 'bar': 42},
    'name': 'Validation with metadata',
}

VALIDATION_GROUPS_1_2_PARSED = {
    'description': 'A validation belonging to groups 1 and 2',
    'groups': ['group1', 'group2'],
    'id': 'VALIDATION_GROUPS_1_2',
    'parameters': {},
    'name': 'Validation from many groups',
}


class ValidationsKeyTest(base.TestCase):

    @mock.patch("oslo_concurrency.processutils.execute")
    @mock.patch('tempfile.mkstemp')
    def test_write_identity_file(self, mock_mkstemp, mock_execute):
        mock_open_context = mock.mock_open()
        mock_mkstemp.return_value = 'fd', 'tmp_path'
        with mock.patch('os.fdopen',
                        mock_open_context):
            validations.write_identity_file('private_key')

        mock_open_context.assert_called_once_with('fd', 'w')
        mock_open_context().write.assert_called_once_with('private_key')
        mock_execute.assert_called_once_with(
            '/usr/bin/sudo', '/usr/bin/chown', '-h', 'validations:',
            'tmp_path')

    @mock.patch("oslo_concurrency.processutils.execute")
    def test_cleanup_identity_file(self, mock_execute):
        validations.cleanup_identity_file('/path/to/key')
        mock_execute.assert_called_once_with(
            '/usr/bin/sudo', '/usr/bin/rm', '-f', '/path/to/key')


class LoadValidationsTest(base.TestCase):

    def test_get_validation_metadata(self):
        validation = yaml.safe_load(VALIDATION_GROUP_1)
        value = validations.get_validation_metadata(validation, 'name')
        self.assertEqual('First validation', value)

    @mock.patch('tripleo_common.utils.validations.DEFAULT_METADATA')
    def test_get_validation_metadata_default_value(self, mock_metadata):
        mock_metadata.get.return_value = 'default_value'
        value = validations.get_validation_metadata({}, 'missing')
        self.assertEqual('default_value', value)

    def test_get_validation_parameters(self):
        validation = yaml.safe_load(VALIDATION_WITH_METADATA)
        value = validations.get_validation_parameters(validation)
        expected = {
            'foo': 'a foo metadata',
            'bar': 42
        }
        self.assertEqual(expected, value)

    def test_get_validation_parameters_no_extra(self):
        validation = yaml.safe_load(VALIDATION_GROUP_1)
        value = validations.get_validation_parameters(validation)
        self.assertEqual({}, value)

    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
    def test_load_validations_no_group(self, mock_get_object_client):
        swiftclient = mock.MagicMock(url='http://swift:8080/v1/AUTH_test')
        swiftclient.get_container.side_effect = (
            ({}, []),  # no custom validations
            ({},
             [{'name': 'VALIDATION_GROUP_1.yaml', 'groups': ['group1']},
              {'name': 'VALIDATION_WITH_METADATA.yaml'}]))
        swiftclient.get_object.side_effect = (
            ({}, VALIDATION_GROUP_1),
            ({}, VALIDATION_WITH_METADATA),
        )
        mock_get_object_client.return_value = swiftclient

        my_validations = validations.load_validations(
            mock_get_object_client(), plan='overcloud')

        expected = [VALIDATION_GROUP_1_PARSED, VALIDATION_WITH_METADATA_PARSED]
        self.assertEqual(expected, my_validations)

    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
    def test_load_validations_group(self, mock_get_object_client):
        swiftclient = mock.MagicMock(url='http://swift:8080/v1/AUTH_test')
        swiftclient.get_container.side_effect = (
            ({}, []),  # no custom validations
            ({},
             [
                {'name': 'VALIDATION_GROUPS_1_2.yaml',
                 'groups': ['group1', 'group2']},
                {'name': 'VALIDATION_GROUP_1.yaml', 'groups': ['group1']},
                {'name': 'VALIDATION_WITH_METADATA.yaml'}
                ]
             )
        )
        swiftclient.get_object.side_effect = (
            ({}, VALIDATION_GROUPS_1_2),
            ({}, VALIDATION_GROUP_1),
            ({}, VALIDATION_WITH_METADATA),
        )
        mock_get_object_client.return_value = swiftclient

        my_validations = validations.load_validations(
            mock_get_object_client(), plan='overcloud', groups=['group1'])

        expected = [VALIDATION_GROUPS_1_2_PARSED, VALIDATION_GROUP_1_PARSED]
        self.assertEqual(expected, my_validations)

    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
    def test_load_validations_custom_gets_picked_over_default(
            self, mock_get_object_client):
        swiftclient = mock.MagicMock(url='http://swift:8080/v1/AUTH_test')
        swiftclient.get_container.side_effect = (
            ({}, [{'name': 'FIRST_VALIDATION.yaml'}]),
            ({}, [{'name': 'FIRST_VALIDATION.yaml'}])
        )
        swiftclient.get_object.side_effect = (
            ({}, VALIDATION_CUSTOM),
            ({}, VALIDATION_DEFAULT)
        )
        mock_get_object_client.return_value = swiftclient

        my_validations = validations.load_validations(
            mock_get_object_client(), plan='overcloud')

        self.assertEqual(len(my_validations), 1)
        self.assertEqual('Custom validation', my_validations[0]['description'])


class RunValidationTest(base.TestCase):

    @mock.patch('tripleo_common.actions.base.TripleOAction.get_object_client')
    @mock.patch('tripleo_common.utils.validations.download_validation')
    @mock.patch('oslo_concurrency.processutils.execute')
    def test_run_validation(self, mock_execute,
                            mock_download_validation, mock_get_object_client):
        swiftclient = mock.MagicMock(url='http://swift:8080/v1/AUTH_test')
        mock_get_object_client.return_value = swiftclient
        Ctx = namedtuple('Ctx', 'auth_uri user_name auth_token project_name')
        mock_ctx = Ctx(
            auth_uri='auth_uri',
            user_name='user_name',
            auth_token='auth_token',
            project_name='project_name'
        )
        mock_execute.return_value = 'output'
        mock_download_validation.return_value = 'validation_path'

        result = validations.run_validation(mock_get_object_client(),
                                            'validation', 'identity_file',
                                            'plan', 'inputs_file', mock_ctx)
        self.assertEqual('output', result)
        mock_execute.assert_called_once_with(
            '/usr/bin/sudo', '-u', 'validations',
            'OS_AUTH_URL=auth_uri',
            'OS_USERNAME=user_name',
            'OS_AUTH_TOKEN=auth_token',
            'OS_TENANT_NAME=project_name',
            '/usr/bin/run-validation',
            '--inputs', 'inputs_file',
            'validation_path',
            'identity_file',
            'plan',
            '/usr/share/openstack-tripleo-validations'
        )
        mock_download_validation.assert_called_once_with(
            mock_get_object_client(), 'plan', 'validation')


class RunPatternValidatorTest(base.TestCase):

    def test_valid_patterns(self):
        self.assertTrue(validations.pattern_validator("^$", ""))
        self.assertTrue(
            validations.pattern_validator(PLAN_NAME_PATTERN, "foo"))
        self.assertTrue(
            validations.pattern_validator(PLAN_NAME_PATTERN, "Foo-1"))

    def test_invalid_patterns(self):
        self.assertFalse(
            validations.pattern_validator("^$", "foo"))
        self.assertFalse(
            validations.pattern_validator(PLAN_NAME_PATTERN, "foo_1"))



# Copyright 2015 Red Hat, Inc.
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


#: The names of the root template in a standard tripleo-heat-template layout.
OVERCLOUD_YAML_NAME = "overcloud.yaml"

#: The name of the overcloud root template in jinja2 format.
OVERCLOUD_J2_NAME = "overcloud.j2.yaml"

#: The name of custom roles data file used when rendering the jinja template.
OVERCLOUD_J2_ROLES_NAME = "roles_data.yaml"

#: The name of custom roles network data file used when rendering j2 templates.
OVERCLOUD_J2_NETWORKS_NAME = "network_data.yaml"

#: The name of custom roles excl file used when rendering the jinja template.
OVERCLOUD_J2_EXCLUDES = "j2_excludes.yaml"

#: The name of the type for resource groups.
RESOURCE_GROUP_TYPE = 'OS::Heat::ResourceGroup'

#: The resource name used for package updates
UPDATE_RESOURCE_NAME = 'UpdateDeployment'

#: The default timeout to pass to Heat stacks
STACK_TIMEOUT_DEFAULT = 240

#: The default name to use for a plan container
DEFAULT_CONTAINER_NAME = 'overcloud'

#: The default name to use for the config files of the container
CONFIG_CONTAINER_NAME = 'overcloud-config'

#: The default name to use for the container for validations
VALIDATIONS_CONTAINER_NAME = 'tripleo-validations'

#: The name of the plan subdirectory that holds custom validations
CUSTOM_VALIDATIONS_FOLDER = 'custom-validations'

#: The default key to use for updating parameters in plan environment.
DEFAULT_PLAN_ENV_KEY = 'parameter_defaults'

#: The path to the tripleo heat templates installed on the undercloud
DEFAULT_TEMPLATES_PATH = '/usr/share/openstack-tripleo-heat-templates/'

#: The path to the base directory of tripleo-validations
DEFAULT_VALIDATIONS_BASEDIR = '/usr/share/openstack-tripleo-validations'

# The path to the tripleo validations installed on the undercloud
DEFAULT_VALIDATIONS_PATH = DEFAULT_VALIDATIONS_BASEDIR + '/playbooks/'

# The path to the local CA certificate installed on the undercloud
LOCAL_CACERT_PATH = '/etc/pki/ca-trust/source/anchors/cm-local-ca.pem'

# TRIPLEO_META_USAGE_KEY is inserted into metadata for containers created in
# Swift via SwiftPlanStorageBackend to identify them from other containers
TRIPLEO_META_USAGE_KEY = 'x-container-meta-usage-tripleo'

# 60 minutes maximum to build the child layers at the same time.
BUILD_TIMEOUT = 3600

#: List of names of parameters that contain passwords
PASSWORD_PARAMETER_NAMES = (
    'AdminPassword',
    'AdminToken',
    'AodhPassword',
    'BarbicanPassword',
    'BarbicanSimpleCryptoKek',
    'CeilometerMeteringSecret',
    'CeilometerPassword',
    'CephClientKey',
    'CephClusterFSID',
    'CephManilaClientKey',
    'CephRgwKey',
    'CephGrafanaAdminPassword',
    'CephDashboardAdminPassword',
    'CinderPassword',
    'CongressPassword',
    'DesignatePassword',
    'DesignateRndcKey',
    'Ec2ApiPassword',
    'EtcdInitialClusterToken',
    'GlancePassword',
    'GnocchiPassword',
    'HAProxyStatsPassword',
    'HeatAuthEncryptionKey',
    'HeatPassword',
    'HeatStackDomainAdminPassword',
    'HorizonSecret',
    'IronicPassword',
    'LibvirtTLSPassword',
    'KeystoneCredential0',
    'KeystoneCredential1',
    'KeystoneFernetKey0',
    'KeystoneFernetKey1',
    'KeystoneFernetKeys',
    'ManilaPassword',
    'MistralPassword',
    'MysqlClustercheckPassword',
    'MysqlRootPassword',
    'NeutronMetadataProxySharedSecret',
    'NeutronPassword',
    'NovaPassword',
    'NovajoinPassword',
    'MigrationSshKey',
    'OctaviaServerCertsKeyPassphrase',
    'OctaviaCaKeyPassphrase',
    'OctaviaHeartbeatKey',
    'OctaviaPassword',
    'PacemakerRemoteAuthkey',
    'PankoPassword',
    'PcsdPassword',
    'PlacementPassword',
    'RpcPassword',
    'NotifyPassword',
    'RabbitCookie',
    'RabbitPassword',
    'RedisPassword',
    'SaharaPassword',
    'SnmpdReadonlyUserPassword',
    'SwiftHashSuffix',
    'SwiftPassword',
    'ZaqarPassword',
)
# List of legacy resource names from overcloud.yaml
LEGACY_HEAT_PASSWORD_RESOURCE_NAMES = (
    'HeatAuthEncryptionKey',
    'HorizonSecret',
    'MysqlRootPassword',
    'PcsdPassword',
    'RabbitCookie',
)

# List of passwords that should not be rotated by default using the
# GeneratePasswordAction because they require some special handling
DO_NOT_ROTATE_LIST = (
    'BarbicanSimpleCryptoKek',
    'KeystoneCredential0',
    'KeystoneCredential1',
    'KeystoneFernetKey0',
    'KeystoneFernetKey1',
    'KeystoneFernetKeys',
    'CephClientKey',
    'CephClusterFSID',
    'CephManilaClientKey',
    'CephRgwKey',
)

PLAN_NAME_PATTERN = '^[a-zA-Z0-9-]+$'

# The default version of the Identity API to set in overcloudrc.
DEFAULT_IDENTITY_API_VERSION = '3'

# The default version of the Compute API to set in overcloudrc.
DEFAULT_COMPUTE_API_VERSION = '2.latest'

# The default version of the Image API to set in overcloudrc.
DEFAULT_IMAGE_API_VERSION = '2'

# The default version of the Volume API to set in overcloudrc.
DEFAULT_VOLUME_API_VERSION = '3'

# The name of the file which holds the Mistral environment contents for plan
# import/export
PLAN_ENVIRONMENT = 'plan-environment.yaml'

# Name of the environment with merged parameters from CLI
USER_ENVIRONMENT = 'user-environment.yaml'

# The name of the file which holds container image default parameters
CONTAINER_DEFAULTS_ENVIRONMENT = ('environments/'
                                  'containers-default-parameters.yaml')

DEFAULT_DEPLOY_KERNEL_NAME = 'bm-deploy-kernel'

DEFAULT_DEPLOY_RAMDISK_NAME = 'bm-deploy-ramdisk'

# The name for the swift container to host the cache for tripleo
TRIPLEO_CACHE_CONTAINER = "__cache__"

TRIPLEO_UI_LOG_FILE_SIZE = 1e7  # 10MB
TRIPLEO_UI_LOG_FILENAME = 'tripleo-ui.logs'

API_NETWORK = 'InternalApi'
LEGACY_API_NETWORK = 'Internal'

# Default nested depth when recursing Heat stacks
NESTED_DEPTH = 7

# Resource name for deployment resources when using config download
TRIPLEO_DEPLOYMENT_RESOURCE = 'TripleODeployment'

# Resource name for network config resources when using config download
TRIPLEO_NETWORK_CONFIG_RESOURCE = 'NetworkConfig'

HOST_NETWORK = 'ctlplane'

EXTERNAL_TASKS = ['external_deploy_tasks']

ANSIBLE_ERRORS_FILE = 'ansible-errors.json'

DEPLOYMENT_STATUS_FILE = 'deployment_status.yaml'

MISTRAL_WORK_DIR = '/var/lib/mistral'

EXCLUSIVE_NEUTRON_DRIVERS = ['ovn', 'openvswitch']

DEFAULT_STEPS_MAX = 6

PER_STEP_TASKS = ['upgrade_tasks', 'deploy_steps_tasks']



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
from unittest import mock

from testtools import matchers

from tripleo_common import exception
from tripleo_common.tests import base
from tripleo_common.utils import nodes


class DriverInfoTest(base.TestCase):
    def setUp(self):
        super(DriverInfoTest, self).setUp()
        self.driver_info = nodes.DriverInfo(
            'foo',
            mapping={
                'pm_1': 'foo_1',
                'pm_2': 'foo_2'
            },
            deprecated_mapping={
                'pm_3': 'foo_3'
            })

    def test_convert_key(self):
        self.assertEqual('foo_1', self.driver_info.convert_key('pm_1'))
        self.assertEqual('foo_42', self.driver_info.convert_key('foo_42'))
        self.assertIsNone(self.driver_info.convert_key('bar_baz'))

    @mock.patch.object(nodes.LOG, 'warning', autospec=True)
    def test_convert_key_deprecated(self, mock_log):
        self.assertEqual('foo_3', self.driver_info.convert_key('pm_3'))
        self.assertTrue(mock_log.called)

    @mock.patch.object(nodes.LOG, 'warning', autospec=True)
    def test_convert_key_pm_unsupported(self, mock_log):
        self.assertIsNone(self.driver_info.convert_key('pm_42'))
        self.assertTrue(mock_log.called)

    def test_convert(self):
        result = self.driver_info.convert({'pm_1': 'val1',
                                           'foo_42': 42,
                                           'unknown': 'foo'})
        self.assertEqual({'foo_1': 'val1', 'foo_42': 42}, result)


class PrefixedDriverInfoTest(base.TestCase):
    def setUp(self):
        super(PrefixedDriverInfoTest, self).setUp()
        self.driver_info = nodes.PrefixedDriverInfo(
            'foo', deprecated_mapping={'pm_d': 'foo_d'})

    def test_convert_key(self):
        keys = {'pm_addr': 'foo_address',
                'pm_user': 'foo_username',
                'pm_password': 'foo_password',
                'foo_something': 'foo_something',
                'pm_d': 'foo_d'}
        for key, expected in keys.items():
            self.assertEqual(expected, self.driver_info.convert_key(key))

        for key in ('unknown', 'pm_port'):
            self.assertIsNone(self.driver_info.convert_key(key))

    def test_unique_id_from_fields(self):
        fields = {'pm_addr': 'localhost',
                  'pm_user': 'user',
                  'pm_password': '123456',
                  'pm_port': 42}
        self.assertEqual('localhost',
                         self.driver_info.unique_id_from_fields(fields))

    def test_unique_id_from_node(self):
        node = mock.Mock(driver_info={'foo_address': 'localhost',
                                      'foo_port': 42})
        self.assertEqual('localhost',
                         self.driver_info.unique_id_from_node(node))


class PrefixedDriverInfoTestWithPort(base.TestCase):
    def setUp(self):
        super(PrefixedDriverInfoTestWithPort, self).setUp()
        self.driver_info = nodes.PrefixedDriverInfo(
            'foo', deprecated_mapping={'pm_d': 'foo_d'},
            has_port=True)

    def test_convert_key_with_port(self):
        keys = {'pm_addr': 'foo_address',
                'pm_user': 'foo_username',
                'pm_password': 'foo_password',
                'foo_something': 'foo_something',
                'pm_d': 'foo_d',
                'pm_port': 'foo_port'}
        for key, expected in keys.items():
            self.assertEqual(expected, self.driver_info.convert_key(key))

        self.assertIsNone(self.driver_info.convert_key('unknown'))

    def test_unique_id_from_fields(self):
        fields = {'pm_addr': 'localhost',
                  'pm_user': 'user',
                  'pm_password': '123456',
                  'pm_port': 42}
        self.assertEqual('localhost:42',
                         self.driver_info.unique_id_from_fields(fields))

    def test_unique_id_from_node(self):
        node = mock.Mock(driver_info={'foo_address': 'localhost',
                                      'foo_port': 42})
        self.assertEqual('localhost:42',
                         self.driver_info.unique_id_from_node(node))


class RedfishDriverInfoTest(base.TestCase):
    driver_info = nodes.RedfishDriverInfo()

    def test_convert_key(self):
        keys = {'pm_addr': 'redfish_address',
                'pm_user': 'redfish_username',
                'pm_password': 'redfish_password',
                'pm_system_id': 'redfish_system_id',
                'redfish_verify_ca': 'redfish_verify_ca'}
        for key, expected in keys.items():
            self.assertEqual(expected, self.driver_info.convert_key(key))

        self.assertIsNone(self.driver_info.convert_key('unknown'))

    def test_unique_id_from_fields(self):
        for address in ['example.com',
                        'http://example.com/',
                        'https://example.com/']:
            fields = {'pm_addr': address,
                      'pm_user': 'user',
                      'pm_password': '123456',
                      'pm_system_id': '/redfish/v1/Systems/1'}
            self.assertEqual('example.com/redfish/v1/Systems/1',
                             self.driver_info.unique_id_from_fields(fields))

    def test_unique_id_from_node(self):
        for address in ['example.com',
                        'http://example.com/',
                        'https://example.com/']:
            node = mock.Mock(driver_info={
                'redfish_address': address,
                'redfish_system_id': '/redfish/v1/Systems/1'})
            self.assertEqual('example.com/redfish/v1/Systems/1',
                             self.driver_info.unique_id_from_node(node))


class oVirtDriverInfoTest(base.TestCase):
    driver_info = nodes.oVirtDriverInfo()

    def test_convert_key(self):
        keys = {'pm_addr': 'ovirt_address',
                'pm_user': 'ovirt_username',
                'pm_password': 'ovirt_password',
                'pm_vm_name': 'ovirt_vm_name',
                'ovirt_insecure': 'ovirt_insecure'}
        for key, expected in keys.items():
            self.assertEqual(expected, self.driver_info.convert_key(key))

        self.assertIsNone(self.driver_info.convert_key('unknown'))

    def test_unique_id_from_fields(self):
        fields = {'pm_addr': 'http://127.0.0.1',
                  'pm_user': 'user',
                  'pm_password': '123456',
                  'pm_vm_name': 'My VM'}
        self.assertEqual('http://127.0.0.1:My VM',
                         self.driver_info.unique_id_from_fields(fields))

    def test_unique_id_from_node(self):
        node = mock.Mock(driver_info={
            'ovirt_address': 'http://127.0.0.1',
            'ovirt_vm_name': 'My VM'})
        self.assertEqual('http://127.0.0.1:My VM',
                         self.driver_info.unique_id_from_node(node))


class iBootDriverInfoTest(base.TestCase):
    def setUp(self):
        super(iBootDriverInfoTest, self).setUp()
        self.driver_info = nodes.iBootDriverInfo()

    def test_unique_id_from_fields(self):
        fields = {'pm_addr': 'localhost',
                  'pm_user': 'user',
                  'pm_password': '123456',
                  'pm_port': 42,
                  'iboot_relay_id': 'r1'}
        self.assertEqual('localhost:42#r1',
                         self.driver_info.unique_id_from_fields(fields))

    def test_unique_id_from_fields_no_relay(self):
        fields = {'pm_addr': 'localhost',
                  'pm_user': 'user',
                  'pm_password': '123456',
                  'pm_port': 42}
        self.assertEqual('localhost:42',
                         self.driver_info.unique_id_from_fields(fields))

    def test_unique_id_from_node(self):
        node = mock.Mock(driver_info={'iboot_address': 'localhost',
                                      'iboot_port': 42,
                                      'iboot_relay_id': 'r1'})
        self.assertEqual('localhost:42#r1',
                         self.driver_info.unique_id_from_node(node))

    def test_unique_id_from_node_no_relay(self):
        node = mock.Mock(driver_info={'iboot_address': 'localhost',
                                      'iboot_port': 42})
        self.assertEqual('localhost:42',
                         self.driver_info.unique_id_from_node(node))


class FindNodeHandlerTest(base.TestCase):
    def test_found(self):
        test = [('fake', 'fake'),
                ('fake_pxe', 'fake'),
                ('pxe_ipmitool', 'ipmi'),
                ('ipmi', 'ipmi'),
                ('pxe_ilo', 'ilo'),
                ('ilo', 'ilo'),
                ('pxe_drac', 'drac'),
                ('idrac', 'drac'),
                ('agent_irmc', 'irmc'),
                ('irmc', 'irmc')]
        for driver, prefix in test:
            handler = nodes._find_node_handler({'pm_type': driver})
            self.assertEqual(prefix, handler._prefix)

    def test_no_driver(self):
        self.assertRaises(exception.InvalidNode,
                          nodes._find_node_handler, {})

    def test_unknown_driver(self):
        self.assertRaises(exception.InvalidNode,
                          nodes._find_node_handler, {'pm_type': 'foobar'})


class NodesTest(base.TestCase):

    def _get_node(self):
        return {'cpu': '1', 'memory': '2048', 'disk': '30', 'arch': 'amd64',
                'ports': [{'address': 'aaa'}], 'pm_addr': 'foo.bar',
                'pm_user': 'test', 'pm_password': 'random', 'pm_type': 'ipmi',
                'name': 'node1', 'capabilities': 'num_nics:6'}

    def test_register_all_nodes_ironic_no_hw_stats(self):
        node_list = [self._get_node()]

        # Remove the hardware stats from the node dictionary
        node_list[0].pop("cpu")
        node_list[0].pop("memory")
        node_list[0].pop("disk")
        node_list[0].pop("arch")

        # Node properties should be created with empty string values for the
        # hardware statistics
        node_properties = {"capabilities": "num_nics:6"}

        ironic = mock.MagicMock()
        new_nodes = nodes.register_all_nodes(node_list, client=ironic)
        self.assertEqual([ironic.node.create.return_value], new_nodes)
        pxe_node_driver_info = {"ipmi_address": "foo.bar",
                                "ipmi_username": "test",
                                "ipmi_password": "random"}
        pxe_node = mock.call(driver="ipmi",
                             name='node1',
                             driver_info=pxe_node_driver_info,
                             resource_class='baremetal',
                             properties=node_properties)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa', physical_network='ctlplane',
                              local_link_connection=None)
        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
        ironic.port.create.assert_has_calls([port_call])

    def test_register_all_nodes(self):
        node_list = [self._get_node()]
        node_list[0]['root_device'] = {"serial": "abcdef"}
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6",
                           "root_device": {"serial": "abcdef"}}
        ironic = mock.MagicMock()
        nodes.register_all_nodes(node_list, client=ironic)
        pxe_node_driver_info = {"ipmi_address": "foo.bar",
                                "ipmi_username": "test",
                                "ipmi_password": "random"}
        pxe_node = mock.call(driver="ipmi",
                             name='node1',
                             driver_info=pxe_node_driver_info,
                             resource_class='baremetal',
                             properties=node_properties)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa', physical_network='ctlplane',
                              local_link_connection=None)
        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
        ironic.port.create.assert_has_calls([port_call])

    def test_register_all_nodes_with_platform(self):
        node_list = [self._get_node()]
        node_list[0]['root_device'] = {"serial": "abcdef"}
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6",
                           "root_device": {"serial": "abcdef"}}
        node_list[0].update({'platform': 'SNB'})
        node_extra = {"tripleo_platform": "SNB"}
        ironic = mock.MagicMock()
        nodes.register_all_nodes(node_list, client=ironic)
        pxe_node_driver_info = {"ipmi_address": "foo.bar",
                                "ipmi_username": "test",
                                "ipmi_password": "random"}
        pxe_node = mock.call(driver="ipmi",
                             name='node1',
                             driver_info=pxe_node_driver_info,
                             resource_class='baremetal',
                             properties=node_properties,
                             extra=node_extra)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa', physical_network='ctlplane',
                              local_link_connection=None)
        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
        ironic.port.create.assert_has_calls([port_call])

    def test_register_all_nodes_kernel_ramdisk(self):
        node_list = [self._get_node()]
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        ironic = mock.MagicMock()
        glance = mock.MagicMock()
        image = collections.namedtuple('image', ['id'])
        glance.images.find.side_effect = (image('kernel-123'),
                                          image('ramdisk-999'))
        nodes.register_all_nodes(node_list, client=ironic,
                                 glance_client=glance, kernel_name='bm-kernel',
                                 ramdisk_name='bm-ramdisk')
        pxe_node_driver_info = {"ipmi_address": "foo.bar",
                                "ipmi_username": "test",
                                "ipmi_password": "random",
                                "deploy_kernel": "kernel-123",
                                "deploy_ramdisk": "ramdisk-999",
                                "rescue_kernel": "kernel-123",
                                "rescue_ramdisk": "ramdisk-999"}
        pxe_node = mock.call(driver="ipmi",
                             name='node1',
                             driver_info=pxe_node_driver_info,
                             resource_class='baremetal',
                             properties=node_properties)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa', physical_network='ctlplane',
                              local_link_connection=None)
        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
        ironic.port.create.assert_has_calls([port_call])

    def test_register_all_nodes_uuid(self):
        node_list = [self._get_node()]
        node_list[0]['uuid'] = 'abcdef'
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        ironic = mock.MagicMock()
        nodes.register_all_nodes(node_list, client=ironic)
        pxe_node_driver_info = {"ipmi_address": "foo.bar",
                                "ipmi_username": "test",
                                "ipmi_password": "random"}
        pxe_node = mock.call(driver="ipmi",
                             name='node1',
                             driver_info=pxe_node_driver_info,
                             properties=node_properties,
                             resource_class='baremetal',
                             uuid="abcdef")
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa', physical_network='ctlplane',
                              local_link_connection=None)
        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
        ironic.port.create.assert_has_calls([port_call])

    def test_register_all_nodes_caps_dict(self):
        node_list = [self._get_node()]
        node_list[0]['capabilities'] = {
            'num_nics': 7
        }
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:7"}
        ironic = mock.MagicMock()
        nodes.register_all_nodes(node_list, client=ironic)
        pxe_node_driver_info = {"ipmi_address": "foo.bar",
                                "ipmi_username": "test",
                                "ipmi_password": "random"}
        pxe_node = mock.call(driver="ipmi",
                             name='node1',
                             driver_info=pxe_node_driver_info,
                             resource_class='baremetal',
                             properties=node_properties)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa', physical_network='ctlplane',
                              local_link_connection=None)
        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
        ironic.port.create.assert_has_calls([port_call])

    def test_register_all_nodes_with_profile(self):
        node_list = [self._get_node()]
        node_list[0]['root_device'] = {"serial": "abcdef"}
        node_list[0]['profile'] = "compute"
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6,profile:compute",
                           "root_device": {"serial": "abcdef"}}
        ironic = mock.MagicMock()
        nodes.register_all_nodes(node_list, client=ironic)
        pxe_node_driver_info = {"ipmi_address": "foo.bar",
                                "ipmi_username": "test",
                                "ipmi_password": "random"}
        pxe_node = mock.call(driver="ipmi",
                             name='node1',
                             driver_info=pxe_node_driver_info,
                             resource_class='baremetal',
                             properties=node_properties)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa', physical_network='ctlplane',
                              local_link_connection=None)
        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
        ironic.port.create.assert_has_calls([port_call])

    def test_register_all_nodes_with_interfaces(self):
        interfaces = {'boot_interface': 'pxe',
                      'console_interface': 'ipmitool-socat',
                      'deploy_interface': 'direct',
                      'inspect_interface': 'inspector',
                      'management_interface': 'ipmitool',
                      'network_interface': 'neutron',
                      'power_interface': 'ipmitool',
                      'raid_interface': 'agent',
                      'rescue_interface': 'agent',
                      'storage_interface': 'cinder',
                      'vendor_interface': 'ipmitool'}

        node_list = [self._get_node()]
        node_list[0].update(interfaces)
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        ironic = mock.MagicMock()
        nodes.register_all_nodes(node_list, client=ironic)
        pxe_node_driver_info = {"ipmi_address": "foo.bar",
                                "ipmi_username": "test",
                                "ipmi_password": "random"}
        pxe_node = mock.call(driver="ipmi",
                             name='node1',
                             driver_info=pxe_node_driver_info,
                             properties=node_properties,
                             resource_class='baremetal',
                             **interfaces)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa', local_link_connection=None,
                              physical_network='ctlplane')
        ironic.node.create.assert_has_calls([pxe_node, mock.ANY])
        ironic.port.create.assert_has_calls([port_call])

    def test_register_update(self):
        interfaces = {'boot_interface': 'pxe',
                      'console_interface': 'ipmitool-socat',
                      'deploy_interface': 'direct',
                      'inspect_interface': 'inspector',
                      'management_interface': 'ipmitool',
                      'network_interface': 'neutron',
                      'power_interface': 'ipmitool',
                      'raid_interface': 'agent',
                      'rescue_interface': 'agent',
                      'storage_interface': 'cinder',
                      'vendor_interface': 'ipmitool'}

        node = self._get_node()
        node.update(interfaces)
        node['root_device'] = {'serial': 'abcdef'}
        ironic = mock.MagicMock()
        node_map = {'mac': {'aaa': 1}}

        def side_effect(*args, **kwargs):
            update_patch = [
                {'path': '/name', 'value': 'node1'},
                {'path': '/driver_info/ipmi_password', 'value': 'random'},
                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                {'path': '/properties/memory_mb', 'value': '2048'},
                {'path': '/properties/local_gb', 'value': '30'},
                {'path': '/properties/cpu_arch', 'value': 'amd64'},
                {'path': '/properties/cpus', 'value': '1'},
                {'path': '/properties/capabilities', 'value': 'num_nics:6'},
                {'path': '/properties/root_device',
                 'value': {'serial': 'abcdef'}},
                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
            for iface, value in interfaces.items():
                update_patch.append({'path': '/%s' % iface, 'value': value})
            for key in update_patch:
                key['op'] = 'add'
            self.assertThat(update_patch,
                            matchers.MatchesSetwise(*(map(matchers.Equals,
                                                          args[1]))))
            return mock.Mock(uuid='uuid1')

        ironic.node.update.side_effect = side_effect
        nodes._update_or_register_ironic_node(node, node_map, client=ironic)
        ironic.node.update.assert_called_once_with(1, mock.ANY)

    def test_register_update_caps_dict(self):
        interfaces = {'boot_interface': 'pxe',
                      'console_interface': 'ipmitool-socat',
                      'deploy_interface': 'direct',
                      'inspect_interface': 'inspector',
                      'management_interface': 'ipmitool',
                      'network_interface': 'neutron',
                      'power_interface': 'ipmitool',
                      'raid_interface': 'agent',
                      'rescue_interface': 'agent',
                      'storage_interface': 'cinder',
                      'vendor_interface': 'ipmitool'}

        node = self._get_node()
        node.update(interfaces)
        node['root_device'] = {'serial': 'abcdef'}
        node['capabilities'] = {'profile': 'compute', 'num_nics': 6}
        ironic = mock.MagicMock()
        node_map = {'mac': {'aaa': 1}}

        def side_effect(*args, **kwargs):
            update_patch = [
                {'path': '/name', 'value': 'node1'},
                {'path': '/driver_info/ipmi_password', 'value': 'random'},
                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                {'path': '/properties/memory_mb', 'value': '2048'},
                {'path': '/properties/local_gb', 'value': '30'},
                {'path': '/properties/cpu_arch', 'value': 'amd64'},
                {'path': '/properties/cpus', 'value': '1'},
                {'path': '/properties/capabilities',
                 'value': 'num_nics:6,profile:compute'},
                {'path': '/properties/root_device',
                 'value': {'serial': 'abcdef'}},
                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
            for iface, value in interfaces.items():
                update_patch.append({'path': '/%s' % iface, 'value': value})
            for key in update_patch:
                key['op'] = 'add'
            self.assertThat(update_patch,
                            matchers.MatchesSetwise(*(map(matchers.Equals,
                                                          args[1]))))
            return mock.Mock(uuid='uuid1')

        ironic.node.update.side_effect = side_effect
        nodes._update_or_register_ironic_node(node, node_map, client=ironic)
        ironic.node.update.assert_called_once_with(1, mock.ANY)

    def test_register_update_profile(self):
        interfaces = {'boot_interface': 'pxe',
                      'console_interface': 'ipmitool-socat',
                      'deploy_interface': 'direct',
                      'inspect_interface': 'inspector',
                      'management_interface': 'ipmitool',
                      'network_interface': 'neutron',
                      'power_interface': 'ipmitool',
                      'raid_interface': 'agent',
                      'rescue_interface': 'agent',
                      'storage_interface': 'cinder',
                      'vendor_interface': 'ipmitool'}

        node = self._get_node()
        node.update(interfaces)
        node['root_device'] = {'serial': 'abcdef'}
        node['profile'] = 'compute'
        ironic = mock.MagicMock()
        node_map = {'mac': {'aaa': 1}}

        def side_effect(*args, **kwargs):
            update_patch = [
                {'path': '/name', 'value': 'node1'},
                {'path': '/driver_info/ipmi_password', 'value': 'random'},
                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                {'path': '/properties/memory_mb', 'value': '2048'},
                {'path': '/properties/local_gb', 'value': '30'},
                {'path': '/properties/cpu_arch', 'value': 'amd64'},
                {'path': '/properties/cpus', 'value': '1'},
                {'path': '/properties/capabilities',
                 'value': 'num_nics:6,profile:compute'},
                {'path': '/properties/root_device',
                 'value': {'serial': 'abcdef'}},
                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
            for iface, value in interfaces.items():
                update_patch.append({'path': '/%s' % iface, 'value': value})
            for key in update_patch:
                key['op'] = 'add'
            self.assertThat(update_patch,
                            matchers.MatchesSetwise(*(map(matchers.Equals,
                                                          args[1]))))
            return mock.Mock(uuid='uuid1')

        ironic.node.update.side_effect = side_effect
        nodes._update_or_register_ironic_node(node, node_map, client=ironic)
        ironic.node.update.assert_called_once_with(1, mock.ANY)

    def test_register_update_with_images(self):
        node = self._get_node()
        node['kernel_id'] = 'image-k'
        node['ramdisk_id'] = 'image-r'
        ironic = mock.MagicMock()
        node_map = {'mac': {'aaa': 1}}

        def side_effect(*args, **kwargs):
            update_patch = [
                {'path': '/name', 'value': 'node1'},
                {'path': '/driver_info/ipmi_password', 'value': 'random'},
                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                {'path': '/properties/memory_mb', 'value': '2048'},
                {'path': '/properties/local_gb', 'value': '30'},
                {'path': '/properties/cpu_arch', 'value': 'amd64'},
                {'path': '/properties/cpus', 'value': '1'},
                {'path': '/properties/capabilities', 'value': 'num_nics:6'},
                {'path': '/driver_info/deploy_kernel', 'value': 'image-k'},
                {'path': '/driver_info/deploy_ramdisk', 'value': 'image-r'},
                {'path': '/driver_info/rescue_kernel', 'value': 'image-k'},
                {'path': '/driver_info/rescue_ramdisk', 'value': 'image-r'},
                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
            for key in update_patch:
                key['op'] = 'add'
            self.assertThat(update_patch,
                            matchers.MatchesSetwise(*(map(matchers.Equals,
                                                          args[1]))))
            return mock.Mock(uuid='uuid1')

        ironic.node.update.side_effect = side_effect
        nodes._update_or_register_ironic_node(node, node_map, client=ironic)
        ironic.node.update.assert_called_once_with(1, mock.ANY)

    def test_register_update_with_interfaces(self):
        node = self._get_node()
        ironic = mock.MagicMock()
        node_map = {'mac': {'aaa': 1}}

        def side_effect(*args, **kwargs):
            update_patch = [
                {'path': '/name', 'value': 'node1'},
                {'path': '/driver_info/ipmi_password', 'value': 'random'},
                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                {'path': '/properties/memory_mb', 'value': '2048'},
                {'path': '/properties/local_gb', 'value': '30'},
                {'path': '/properties/cpu_arch', 'value': 'amd64'},
                {'path': '/properties/cpus', 'value': '1'},
                {'path': '/properties/capabilities', 'value': 'num_nics:6'},
                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
            for key in update_patch:
                key['op'] = 'add'
            self.assertThat(update_patch,
                            matchers.MatchesSetwise(*(map(matchers.Equals,
                                                          args[1]))))
            return mock.Mock(uuid='uuid1')

        ironic.node.update.side_effect = side_effect
        nodes._update_or_register_ironic_node(node, node_map, client=ironic)
        ironic.node.update.assert_called_once_with(1, mock.ANY)

    def _update_by_type(self, pm_type):
        ironic = mock.MagicMock()
        node_map = {'mac': {}, 'pm_addr': {}}
        node = self._get_node()
        node['pm_type'] = pm_type
        node_map['pm_addr']['foo.bar'] = ironic.node.get.return_value.uuid
        nodes._update_or_register_ironic_node(node, node_map, client=ironic)
        ironic.node.update.assert_called_once_with(
            ironic.node.get.return_value.uuid, mock.ANY)

    def test_update_node_ironic_ipmi(self):
        self._update_by_type('ipmi')

    def test_update_node_ironic_pxe_ipmitool(self):
        self._update_by_type('pxe_ipmitool')

    def test_update_node_ironic_idrac(self):
        self._update_by_type('idrac')

    def test_update_node_ironic_pxe_drac(self):
        self._update_by_type('pxe_drac')

    def test_update_node_ironic_ilo(self):
        self._update_by_type('ilo')

    def test_update_node_ironic_pxe_ilo(self):
        self._update_by_type('pxe_ilo')

    def test_update_node_ironic_irmc(self):
        self._update_by_type('irmc')

    def test_update_node_ironic_pxe_irmc(self):
        self._update_by_type('pxe_irmc')

    def test_update_node_ironic_xclarity(self):
        self._update_by_type('xclarity')

    def test_update_node_ironic_redfish(self):
        ironic = mock.MagicMock()
        node_map = {'mac': {}, 'pm_addr': {}}
        node = self._get_node()
        node.update({'pm_type': 'redfish',
                     'pm_system_id': '/path'})
        node_map['pm_addr']['foo.bar/path'] = ironic.node.get.return_value.uuid
        nodes._update_or_register_ironic_node(node, node_map, client=ironic)
        ironic.node.update.assert_called_once_with(
            ironic.node.get.return_value.uuid, mock.ANY)

    def test_update_node_ironic_ovirt(self):
        ironic = mock.MagicMock()
        node_map = {'mac': {}, 'pm_addr': {}}
        node = self._get_node()
        node.update({'pm_type': 'staging-ovirt',
                     'pm_vm_name': 'VM1'})
        node_map['pm_addr']['foo.bar:VM1'] = ironic.node.get.return_value.uuid
        nodes._update_or_register_ironic_node(node, node_map, client=ironic)
        ironic.node.update.assert_called_once_with(
            ironic.node.get.return_value.uuid, mock.ANY)

    def test_register_node_update(self):
        node = self._get_node()
        node['ports'][0]['address'] = node['ports'][0]['address'].upper()
        ironic = mock.MagicMock()
        node_map = {'mac': {'aaa': 1}}

        def side_effect(*args, **kwargs):
            update_patch = [
                {'path': '/name', 'value': 'node1'},
                {'path': '/driver_info/ipmi_password', 'value': 'random'},
                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                {'path': '/properties/memory_mb', 'value': '2048'},
                {'path': '/properties/local_gb', 'value': '30'},
                {'path': '/properties/cpu_arch', 'value': 'amd64'},
                {'path': '/properties/cpus', 'value': '1'},
                {'path': '/properties/capabilities', 'value': 'num_nics:6'},
                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
            for key in update_patch:
                key['op'] = 'add'
            self.assertThat(update_patch,
                            matchers.MatchesSetwise(*(map(matchers.Equals,
                                                          args[1]))))
            return mock.Mock(uuid='uuid1')

        ironic.node.update.side_effect = side_effect
        nodes._update_or_register_ironic_node(node, node_map, client=ironic)
        ironic.node.update.assert_called_once_with(1, mock.ANY)

    def test_register_node_update_with_uuid(self):
        node = self._get_node()
        node['uuid'] = 'abcdef'
        ironic = mock.MagicMock()
        node_map = {'uuids': {'abcdef'}}

        def side_effect(*args, **kwargs):
            update_patch = [
                {'path': '/name', 'value': 'node1'},
                {'path': '/driver_info/ipmi_password', 'value': 'random'},
                {'path': '/driver_info/ipmi_address', 'value': 'foo.bar'},
                {'path': '/properties/memory_mb', 'value': '2048'},
                {'path': '/properties/local_gb', 'value': '30'},
                {'path': '/properties/cpu_arch', 'value': 'amd64'},
                {'path': '/properties/cpus', 'value': '1'},
                {'path': '/properties/capabilities', 'value': 'num_nics:6'},
                {'path': '/driver_info/ipmi_username', 'value': 'test'}]
            for key in update_patch:
                key['op'] = 'add'
            self.assertThat(update_patch,
                            matchers.MatchesSetwise(*(map(matchers.Equals,
                                                          args[1]))))
            return mock.Mock(uuid='abcdef')

        ironic.node.update.side_effect = side_effect
        nodes._update_or_register_ironic_node(node, node_map, client=ironic)
        ironic.node.update.assert_called_once_with('abcdef', mock.ANY)

    def test_register_ironic_node_fake_pxe(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        for v in ('pm_addr', 'pm_user', 'pm_password'):
            del node[v]
        node['pm_type'] = 'fake_pxe'
        client = mock.MagicMock()
        nodes.register_ironic_node(node, client=client)
        client.node.create.assert_called_once_with(driver='manual-management',
                                                   name='node1',
                                                   properties=node_properties,
                                                   resource_class='baremetal',
                                                   driver_info={})

    def test_register_ironic_node_conductor_group(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        node['conductor_group'] = 'cg1'
        client = mock.MagicMock()
        nodes.register_ironic_node(node, client=client)
        client.node.create.assert_called_once_with(
            driver='ipmi', name='node1',
            properties=node_properties,
            resource_class='baremetal',
            driver_info={'ipmi_password': 'random', 'ipmi_address': 'foo.bar',
                         'ipmi_username': 'test'},
            conductor_group='cg1')

    def test_register_ironic_node_ipmi(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        node['pm_type'] = 'ipmi'
        node['pm_port'] = '6230'
        client = mock.MagicMock()
        nodes.register_ironic_node(node, client=client)
        client.node.create.assert_called_once_with(
            driver='ipmi', name='node1', properties=node_properties,
            resource_class='baremetal',
            driver_info={'ipmi_password': 'random', 'ipmi_address': 'foo.bar',
                         'ipmi_username': 'test', 'ipmi_port': '6230'})

    def test_register_ironic_node_pxe_ipmitool(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        node['pm_type'] = 'pxe_ipmitool'
        node['pm_port'] = '6230'
        client = mock.MagicMock()
        nodes.register_ironic_node(node, client=client)
        client.node.create.assert_called_once_with(
            driver='ipmi', name='node1', properties=node_properties,
            resource_class='baremetal',
            driver_info={'ipmi_password': 'random', 'ipmi_address': 'foo.bar',
                         'ipmi_username': 'test', 'ipmi_port': '6230'})

    def test_register_ironic_node_idrac(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        node['pm_type'] = 'idrac'
        node['pm_port'] = '6230'
        client = mock.MagicMock()
        nodes.register_ironic_node(node, client=client)
        client.node.create.assert_called_once_with(
            driver='idrac', name='node1', properties=node_properties,
            resource_class='baremetal',
            driver_info={'drac_password': 'random', 'drac_address': 'foo.bar',
                         'drac_username': 'test', 'drac_port': '6230'})

    def test_register_ironic_node_ilo(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        node['pm_type'] = 'ilo'
        node['pm_port'] = '1234'
        client = mock.MagicMock()
        nodes.register_ironic_node(node, client=client)
        client.node.create.assert_called_once_with(
            driver='ilo', name='node1', properties=node_properties,
            resource_class='baremetal',
            driver_info={'ilo_password': 'random', 'ilo_address': 'foo.bar',
                         'ilo_username': 'test', 'ilo_port': '1234'})

    def test_register_ironic_node_pxe_drac(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        node['pm_type'] = 'pxe_drac'
        node['pm_port'] = '6230'
        client = mock.MagicMock()
        nodes.register_ironic_node(node, client=client)
        client.node.create.assert_called_once_with(
            driver='idrac', name='node1', properties=node_properties,
            resource_class='baremetal',
            driver_info={'drac_password': 'random', 'drac_address': 'foo.bar',
                         'drac_username': 'test', 'drac_port': '6230'})

    def test_register_ironic_node_pxe_ilo(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        node['pm_type'] = 'pxe_ilo'
        node['pm_port'] = '1234'
        client = mock.MagicMock()
        nodes.register_ironic_node(node, client=client)
        client.node.create.assert_called_once_with(
            driver='ilo', name='node1', properties=node_properties,
            resource_class='baremetal',
            driver_info={'ilo_password': 'random', 'ilo_address': 'foo.bar',
                         'ilo_username': 'test', 'ilo_port': '1234'})

    def test_register_ironic_node_redfish(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        node['pm_type'] = 'redfish'
        node['pm_system_id'] = '/redfish/v1/Systems/1'
        client = mock.MagicMock()
        nodes.register_ironic_node(node, client=client)
        client.node.create.assert_called_once_with(
            driver='redfish', name='node1', properties=node_properties,
            resource_class='baremetal',
            driver_info={'redfish_password': 'random',
                         'redfish_address': 'foo.bar',
                         'redfish_username': 'test',
                         'redfish_system_id': '/redfish/v1/Systems/1'})

    def test_register_ironic_node_redfish_without_credentials(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        node['pm_type'] = 'redfish'
        node['pm_system_id'] = '/redfish/v1/Systems/1'
        del node['pm_user']
        del node['pm_password']
        client = mock.MagicMock()
        nodes.register_ironic_node(node, client=client)
        client.node.create.assert_called_once_with(
            driver='redfish', name='node1', properties=node_properties,
            resource_class='baremetal',
            driver_info={'redfish_address': 'foo.bar',
                         'redfish_system_id': '/redfish/v1/Systems/1'})

    def test_register_ironic_node_with_physical_network(self):
        node = self._get_node()
        node['ports'] = [{'physical_network': 'subnet1', 'address': 'aaa'}]
        ironic = mock.MagicMock()
        nodes.register_ironic_node(node, client=ironic)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa', physical_network='subnet1',
                              local_link_connection=None)
        ironic.port.create.assert_has_calls([port_call])

    def test_register_ironic_node_with_local_link_connection(self):
        node = self._get_node()
        node['ports'] = [
            {
                'local_link_connection': {
                    "switch_info": "switch",
                    "port_id": "port1",
                    "switch_id": "bbb"
                },
                'physical_network': 'subnet1',
                'address': 'aaa'
            }
        ]
        ironic = mock.MagicMock()
        nodes.register_ironic_node(node, client=ironic)
        port_call = mock.call(node_uuid=ironic.node.create.return_value.uuid,
                              address='aaa', physical_network='subnet1',
                              local_link_connection={"switch_info": "switch",
                                                     "port_id": "port1",
                                                     "switch_id": "bbb"})
        ironic.port.create.assert_has_calls([port_call])

    def test_clean_up_extra_nodes_ironic(self):
        node = collections.namedtuple('node', ['uuid'])
        client = mock.MagicMock()
        client.node.list.return_value = [node('foobar')]
        seen = [node('abcd')]
        nodes._clean_up_extra_nodes(seen, client, remove=True)
        client.node.delete.assert_called_once_with('foobar')

    def test__get_node_id_manual_management(self):
        node = self._get_node()
        node['pm_type'] = 'manual-management'
        handler = nodes.find_driver_handler('manual-management')
        node_map = {'mac': {'aaa': 'abcdef'}, 'pm_addr': {}}
        self.assertEqual('abcdef', nodes._get_node_id(node, handler, node_map))

    def test__get_node_id_conflict(self):
        node = self._get_node()
        handler = nodes.find_driver_handler('ipmi')
        node_map = {'mac': {'aaa': 'abcdef'},
                    'pm_addr': {'foo.bar': 'defabc'}}
        self.assertRaises(exception.InvalidNode,
                          nodes._get_node_id,
                          node, handler, node_map)

    def test_get_node_id_valid_duplicate(self):
        node = self._get_node()
        handler = nodes.find_driver_handler('ipmi')
        node_map = {'mac': {'aaa': 'id'},
                    'pm_addr': {'foo.bar': 'id'}}
        self.assertEqual('id', nodes._get_node_id(node, handler, node_map))

    def test_register_ironic_node_xclarity(self):
        node_properties = {"cpus": "1",
                           "memory_mb": "2048",
                           "local_gb": "30",
                           "cpu_arch": "amd64",
                           "capabilities": "num_nics:6"}
        node = self._get_node()
        node['pm_type'] = 'xclarity'
        node['pm_port'] = '4444'
        client = mock.MagicMock()
        nodes.register_ironic_node(node, client=client)
        client.node.create.assert_called_once_with(
            driver='xclarity', name='node1', properties=node_properties,
            resource_class='baremetal',
            driver_info={'xclarity_password': 'random',
                         'xclarity_address': 'foo.bar',
                         'xclarity_username': 'test',
                         'xclarity_port': '4444'})


class TestPopulateNodeMapping(base.TestCase):
    def test_populate_node_mapping_ironic(self):
        client = mock.MagicMock()
        ironic_node = collections.namedtuple('node', ['uuid', 'driver',
                                             'driver_info'])
        ironic_port = collections.namedtuple('port', ['address'])
        node1 = ironic_node('abcdef', 'redfish', {})
        node2 = ironic_node('fedcba', 'pxe_ipmitool',
                            {'ipmi_address': '10.0.1.2'})
        node3 = ironic_node('xyz', 'ipmi', {'ipmi_address': '10.0.1.3'})
        client.node.list_ports.side_effect = ([ironic_port('aaa')], [], [])
        client.node.list.return_value = [node1, node2, node3]
        expected = {'mac': {'aaa': 'abcdef'},
                    'pm_addr': {'10.0.1.2': 'fedcba', '10.0.1.3': 'xyz'},
                    'uuids': {'abcdef', 'fedcba', 'xyz'}}
        self.assertEqual(expected, nodes._populate_node_mapping(client))

    def test_populate_node_mapping_ironic_manual_management(self):
        client = mock.MagicMock()
        ironic_node = collections.namedtuple('node', ['uuid', 'driver',
                                             'driver_info'])
        ironic_port = collections.namedtuple('port', ['address'])
        node = ironic_node('abcdef', 'manual-management', None)
        client.node.list_ports.return_value = [ironic_port('aaa')]
        client.node.list.return_value = [node]
        expected = {'mac': {'aaa': 'abcdef'}, 'pm_addr': {},
                    'uuids': {'abcdef'}}
        self.assertEqual(expected, nodes._populate_node_mapping(client))


VALID_NODE_JSON = [
    {'_comment': 'This is a comment',
     'pm_type': 'pxe_ipmitool',
     'pm_addr': '192.168.0.1',
     'pm_user': 'root',
     'pm_password': 'p@$$w0rd'},
    {'pm_type': 'ipmi',
     'pm_addr': '192.168.1.1',
     'pm_user': 'root',
     'pm_password': 'p@$$w0rd'},
    {'pm_type': 'pxe_ipmitool',
     'pm_addr': '192.168.0.1',
     'pm_user': 'root',
     'pm_password': 'p@$$w0rd',
     'pm_port': 1234,
     'ipmi_priv_level': 'USER',
     'ports': [
         {'address': 'aa:bb:cc:dd:ee:ff'},
         {'address': '11:22:33:44:55:66'}
     ],
     'name': 'foobar1',
     'capabilities': {'foo': 'bar'},
     'kernel_id': 'kernel1',
     'ramdisk_id': 'ramdisk1'},
    {'pm_type': 'ipmi',
     'pm_addr': '192.168.1.1',
     'pm_user': 'root',
     'pm_password': 'p@$$w0rd',
     'pm_port': 1234,
     'ipmi_priv_level': 'USER',
     'ports': [
         {'address': 'dd:ee:ff:aa:bb:cc'},
         {'address': '44:55:66:11:22:33'}
     ],
     'name': 'foobar2',
     'capabilities': {'foo': 'bar'},
     'kernel_id': 'kernel1',
     'ramdisk_id': 'ramdisk1'},
    {'pm_type': 'idrac',
     'pm_addr': '1.2.3.4',
     'pm_user': 'root',
     'pm_password': 'p@$$w0rd',
     'ports': [
         {'address': '22:22:22:22:22:22'}
     ],
     'capabilities': 'foo:bar,foo1:bar1',
     'cpu': 2,
     'memory': 1024,
     'disk': 40,
     'arch': 'x86_64',
     'root_device': {'foo': 'bar'}},
    {'pm_type': 'redfish',
     'pm_addr': '1.2.3.4',
     'pm_user': 'root',
     'pm_password': 'foobar',
     'pm_system_id': '/redfish/v1/Systems/1'},
    {'pm_type': 'ipmi',
     'pm_addr': '1.1.1.1',
     'pm_user': 'root',
     'pm_password': 'p@$$w0rd',
     'arch': 'x86_64',
     'platform': 'SNB'},
]


class TestValidateNodes(base.TestCase):
    def test_valid(self):
        nodes.validate_nodes(VALID_NODE_JSON)

    def test_unknown_driver(self):
        nodes_json = [
            {'pm_type': 'pxe_foobar',
             'pm_addr': '1.1.1.1',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd'},
        ]
        self.assertRaisesRegex(exception.InvalidNode,
                               'unknown pm_type .* pxe_foobar',
                               nodes.validate_nodes, nodes_json)

    def test_duplicate_ipmi_address(self):
        nodes_json = [
            {'pm_type': 'pxe_ipmitool',
             'pm_addr': '1.1.1.1',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd'},
            {'pm_type': 'ipmi',
             'pm_addr': '1.1.1.1',
             'pm_user': 'user',
             'pm_password': 'p@$$w0rd'},
        ]
        self.assertRaisesRegex(exception.InvalidNode,
                               'Node identified by 1.1.1.1 is already present',
                               nodes.validate_nodes, nodes_json)

    def test_invalid_mac(self):
        nodes_json = [
            {'pm_type': 'pxe_ipmitool',
             'pm_addr': '1.1.1.1',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd',
             'ports': [
                 {'address': '42'}]
             },
        ]
        self.assertRaisesRegex(exception.InvalidNode,
                               'MAC address 42 is invalid',
                               nodes.validate_nodes, nodes_json)

    def test_duplicate_mac(self):
        nodes_json = [
            {'pm_type': 'pxe_ipmitool',
             'pm_addr': '1.1.1.1',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd',
             'ports': [
                 {'address': '11:22:33:44:55:66'}
             ]},
            {'pm_type': 'ipmi',
             'pm_addr': '1.2.1.1',
             'pm_user': 'user',
             'pm_password': 'p@$$w0rd',
             'ports': [
                 {'address': '11:22:33:44:55:66'}
             ]},
        ]
        self.assertRaisesRegex(exception.InvalidNode,
                               'MAC 11:22:33:44:55:66 is not unique',
                               nodes.validate_nodes, nodes_json)

    def test_duplicate_names(self):
        nodes_json = [
            {'pm_type': 'pxe_ipmitool',
             'pm_addr': '1.1.1.1',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd',
             'name': 'name'},
            {'pm_type': 'pxe_ipmitool',
             'pm_addr': '1.2.1.2',
             'pm_user': 'user',
             'pm_password': 'p@$$w0rd',
             'name': 'name'},
        ]
        self.assertRaisesRegex(exception.InvalidNode,
                               'Name "name" is not unique',
                               nodes.validate_nodes, nodes_json)

    def test_invalid_capability(self):
        nodes_json = [
            {'pm_type': 'pxe_ipmitool',
             'pm_addr': '1.1.1.1',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd',
             'capabilities': '42'},
        ]
        self.assertRaisesRegex(exception.InvalidNode,
                               'Invalid capabilities: 42',
                               nodes.validate_nodes, nodes_json)

    def test_unexpected_fields(self):
        nodes_json = [
            {'pm_type': 'pxe_ipmitool',
             'pm_addr': '1.1.1.1',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd',
             'pm_foobar': '42'},
        ]
        self.assertRaisesRegex(exception.InvalidNode,
                               'Unknown field pm_foobar',
                               nodes.validate_nodes, nodes_json)

    def test_missing_fields(self):
        for field in ('pm_addr', 'pm_user', 'pm_password'):
            # NOTE(tonyb): We can't use ipmi here as it's fine with some of
            # these fields being missing.
            nodes_json = [
                {'pm_type': 'pxe_drac',
                 'pm_addr': '1.1.1.1',
                 'pm_user': 'root',
                 'pm_password': 'p@$$w0rd'},
            ]
            del nodes_json[0][field]

            self.assertRaisesRegex(exception.InvalidNode,
                                   'fields are missing: %s' % field,
                                   nodes.validate_nodes, nodes_json)

    def test_missing_arch_with_platform_fail(self):
        nodes_json = [
            {'pm_type': 'ipmi',
             'pm_addr': '1.1.1.1',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd',
             'platform': 'SNB'},
        ]

        msg = 'You have specified a platform without an architecture'
        self.assertRaisesRegex(exception.InvalidNode,
                               msg,
                               nodes.validate_nodes, nodes_json)

    def test_ipmi_missing_user_ok(self):
        nodes_json = [
            {'pm_type': 'ipmi',
             'pm_addr': '1.1.1.1',
             'pm_password': 'p@$$w0rd'},
        ]

        # validate_nodes() doesn't have an explicit return which means python
        # gives us None
        self.assertEqual(None, nodes.validate_nodes(nodes_json))

    def test_duplicate_redfish_node(self):
        nodes_json = [
            {'pm_type': 'redfish',
             'pm_addr': 'example.com',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd',
             'pm_system_id': '/redfish/v1/Systems/1'},
            {'pm_type': 'redfish',
             'pm_addr': 'https://example.com',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd',
             'pm_system_id': '/redfish/v1/Systems/1'},
        ]
        self.assertRaisesRegex(
            exception.InvalidNode,
            'Node identified by example.com/redfish/v1/Systems/1 '
            'is already present',
            nodes.validate_nodes, nodes_json)

    def test_redfish_missing_system_id(self):
        nodes_json = [
            {'pm_type': 'redfish',
             'pm_addr': '1.1.1.1',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd'},
        ]

        self.assertRaisesRegex(exception.InvalidNode,
                               'fields are missing: pm_system_id',
                               nodes.validate_nodes, nodes_json)

    def test_invalid_root_device(self):
        nodes_json = [
            {'pm_type': 'pxe_ipmitool',
             'pm_addr': '1.1.1.1',
             'pm_user': 'root',
             'pm_password': 'p@$$w0rd',
             'root_device': 42}
        ]
        self.assertRaisesRegex(exception.InvalidNode,
                               'Invalid root device',
                               nodes.validate_nodes, nodes_json)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import datetime
import fixtures
import os
from unittest import mock
from unittest.mock import patch
from unittest.mock import call
import uuid
import warnings

import yaml


from tripleo_common import constants
from tripleo_common.tests import base
from tripleo_common.tests.fake_config import fakes
from tripleo_common.utils import config as ooo_config
from tripleo_common.utils.safe_import import git

RESOURCES_YAML_CONTENTS = """heat_template_version: 2016-04-08
resources:
  Controller:
    type: OS::Heat::ResourceGroup
  NotRoleContoller:
    type: OS::Dummy::DummyGroup
"""


class TestConfig(base.TestCase):

    def setUp(self):
        super(TestConfig, self).setUp()

    @patch.object(ooo_config.Config, 'initialize_git_repo')
    @patch.object(ooo_config.shutil, 'copyfile')
    @patch.object(ooo_config.Config, '_mkdir')
    @patch.object(ooo_config.Config, '_open_file')
    @patch.object(ooo_config.shutil, 'rmtree')
    def test_overcloud_config_generate_config(self,
                                              mock_rmtree,
                                              mock_open,
                                              mock_mkdir,
                                              mock_copyfile,
                                              mock_git_init):
        config_type_list = ['config_settings', 'global_config_settings',
                            'logging_sources', 'monitoring_subscriptions',
                            'service_config_settings',
                            'service_metadata_settings',
                            'service_names',
                            'upgrade_batch_tasks', 'upgrade_tasks',
                            'external_deploy_tasks']

        heat = mock.MagicMock()
        heat.stacks.get.return_value = fakes.create_tht_stack()
        self.config = ooo_config.Config(heat)
        self.config.fetch_config('overcloud')
        fake_role = list(self.config.stack_outputs.get('RoleData'))
        self.config.download_config('overcloud', '/tmp/tht', config_type_list)

        mock_git_init.assert_called_once_with('/tmp/tht')
        expected_mkdir_calls = [call('/tmp/tht/%s' % r) for r in fake_role]
        mock_mkdir.assert_has_calls(expected_mkdir_calls, any_order=True)
        mock_mkdir.assert_called()
        expected_calls = []
        for config in config_type_list:
            for role in fake_role:
                if 'external' in config:
                    continue
                elif config == 'step_config':
                    expected_calls += [call('/tmp/tht/%s/%s.pp' %
                                            (role, config))]
                elif config == 'param_config':
                    expected_calls += [call('/tmp/tht/%s/%s.json' %
                                            (role, config))]
                else:
                    expected_calls += [call('/tmp/tht/%s/%s.yaml' %
                                            (role, config))]
        mock_open.assert_has_calls(expected_calls, any_order=True)

    @patch.object(ooo_config.Config, 'initialize_git_repo')
    @patch.object(ooo_config.shutil, 'copyfile')
    @patch.object(ooo_config.Config, '_mkdir')
    @patch.object(ooo_config.Config, '_open_file')
    @patch.object(ooo_config.shutil, 'rmtree')
    def test_overcloud_config_one_config_type(self,
                                              mock_rmtree,
                                              mock_open,
                                              mock_mkdir,
                                              mock_copyfile,
                                              mock_git_init):

        expected_config_type = 'config_settings'

        heat = mock.MagicMock()
        heat.stacks.get.return_value = fakes.create_tht_stack()
        self.config = ooo_config.Config(heat)
        self.config.fetch_config('overcloud')
        fake_role = list(self.config.stack_outputs.get('RoleData'))
        self.config.download_config('overcloud', '/tmp/tht',
                                    ['config_settings'])
        expected_mkdir_calls = [call('/tmp/tht/%s' % r) for r in fake_role]
        expected_calls = [call('/tmp/tht/%s/%s.yaml'
                          % (r, expected_config_type))
                          for r in fake_role]
        mock_mkdir.assert_has_calls(expected_mkdir_calls, any_order=True)
        mock_mkdir.assert_called()
        mock_open.assert_has_calls(expected_calls, any_order=True)
        mock_git_init.assert_called_once_with('/tmp/tht')

    @patch.object(ooo_config.git, 'Repo')
    @mock.patch('os.mkdir')
    @mock.patch('six.moves.builtins.open')
    @patch.object(ooo_config.shutil, 'rmtree')
    def test_overcloud_config_wrong_config_type(self, mock_rmtree,
                                                mock_open, mock_mkdir,
                                                mock_repo):
        args = {'name': 'overcloud', 'config_dir': '/tmp/tht',
                'config_type': ['bad_config']}
        heat = mock.MagicMock()
        heat.stacks.get.return_value = fakes.create_tht_stack()
        self.config = ooo_config.Config(heat)
        self.assertRaises(
            KeyError,
            self.config.download_config, *args)

    def test_overcloud_config_upgrade_tasks(self):

        heat = mock.MagicMock()
        heat.stacks.get.return_value = fakes.create_tht_stack()
        self.config = ooo_config.Config(heat)
        self.config.fetch_config('overcloud')
        self.tmp_dir = self.useFixture(fixtures.TempDir()).path
        fake_role = list(self.config.stack_outputs.get('RoleData'))
        expected_tasks = {'FakeController': {0: [],
                                             1: [{'name': 'Stop fake service',
                                                  'service': 'name=fake '
                                                  'state=stopped',
                                                  'when': 'step|int == 1'}],
                                             2: [],
                                             3: [],
                                             4: [],
                                             5: []},
                          'FakeCompute': {0: [],
                                          1: [{'name': 'Stop fake service',
                                               'service': 'name=fake '
                                               'state=stopped',
                                               'when': ['nova_api_enabled.rc'
                                                        ' == 0', False,
                                                        'httpd_enabled.rc'
                                                        ' != 0',
                                                        'step|int == 1']}],
                                          2: [{'name': 'Stop nova-compute '
                                               'service',
                                               'service': 'name=openstack-'
                                               'nova-compute state=stopped',
                                               'when': ['nova_compute_'
                                                        'enabled.rc == 0',
                                                        'step|int == 2',
                                                        'existing',
                                                        'list']}],
                                          3: [],
                                          4: [],
                                          5: []}}
        for role in fake_role:
            filedir = os.path.join(self.tmp_dir, role)
            os.makedirs(filedir)
            for step in range(constants.DEFAULT_STEPS_MAX):
                filepath = os.path.join(filedir, "upgrade_tasks_step%s.yaml"
                                        % step)
                playbook_tasks = self.config._write_tasks_per_step(
                    self.config.stack_outputs.get('RoleData')[role]
                    ['upgrade_tasks'], role, filepath, step)
                self.assertTrue(os.path.isfile(filepath))
                self.assertEqual(expected_tasks[role][step], playbook_tasks)

    def test_get_server_names(self):
        heat = mock.MagicMock()
        self.config = ooo_config.Config(heat)
        self.config.stack_outputs = {
            'RoleNetHostnameMap': {
                'Controller': {
                    'ctlplane': [
                        'c0.ctlplane.localdomain',
                        'c1.ctlplane.localdomain',
                        'c2.ctlplane.localdomain']}},
            'ServerIdData': {
                'server_ids': {
                    'Controller': [
                        '8269f736',
                        '2af0a373',
                        'c8479674']}}}
        server_names = self.config.get_server_names()
        expected = {'2af0a373': 'c1', '8269f736': 'c0', 'c8479674': 'c2'}
        self.assertEqual(expected, server_names)

    def test_get_role_config(self):
        heat = mock.MagicMock()
        self.config = ooo_config.Config(heat)
        self.config.stack_outputs = {'RoleConfig': None}
        role_config = self.config.get_role_config()
        self.assertEqual({}, role_config)

    def test_get_deployment_data(self):
        heat = mock.MagicMock()
        self.config = ooo_config.Config(heat)
        stack = 'overcloud'
        first = mock.MagicMock()
        first.creation_time = datetime.datetime.now() - datetime.timedelta(2)
        second = mock.MagicMock()
        second.creation_time = datetime.datetime.now() - datetime.timedelta(1)
        third = mock.MagicMock()
        third.creation_time = datetime.datetime.now()
        # Set return_value in a nonsorted order, as we expect the function to
        # sort, so that's what we want to test
        heat.resources.list.return_value = [second, third, first]

        deployment_data = self.config.get_deployment_data(stack)
        self.assertTrue(heat.resources.list.called)
        self.assertEqual(
            heat.resources.list.call_args,
            mock.call(stack,
                      filters=dict(name=constants.TRIPLEO_DEPLOYMENT_RESOURCE),
                      nested_depth=constants.NESTED_DEPTH,
                      with_detail=True))
        self.assertEqual(deployment_data,
                         [first, second, third])

    def _get_config_data(self, datafile):
        config_data_path = os.path.join(
            os.path.dirname(os.path.realpath(__file__)),
            'data',
            datafile)
        with open(config_data_path) as fin:
            config_data = yaml.safe_load(fin.read())
        deployment_data = []

        for deployment in config_data['deployments']:
            deployment_mock = mock.MagicMock()
            deployment_mock.id = deployment['deployment']
            deployment_mock.attributes = dict(
                value=dict(server=deployment['server'],
                           deployment=deployment['deployment'],
                           config=deployment['config'],
                           name=deployment['name']))
            deployment_data.append(deployment_mock)

        configs = config_data['configs']

        return deployment_data, configs

    def _get_deployment_id(self, deployment):
        return deployment.attributes['value']['deployment']

    def _get_config_dict(self, deployment_id):
        deployment = list(filter(
            lambda d: d.id == deployment_id, self.deployments))[0]
        config = self.configs[deployment.attributes['value']['config']].copy()
        config['inputs'] = []
        config['inputs'].append(dict(
            name='deploy_server_id',
            value=deployment.attributes['value']['server']))
        return config

    def _get_yaml_file(self, file_name):
        file_path = os.path.join(
            os.path.dirname(os.path.realpath(__file__)),
            'data',
            file_name)
        with open(file_path) as fin:
            return yaml.safe_load(fin.read())

    @patch.object(ooo_config.Config, 'initialize_git_repo')
    @patch('tripleo_common.utils.config.Config.get_deployment_resource_id')
    @patch('tripleo_common.utils.config.Config.get_config_dict')
    @patch('tripleo_common.utils.config.Config.get_deployment_data')
    def test_config_download(self, mock_deployment_data, mock_config_dict,
                             mock_deployment_resource_id,
                             mock_git_init):
        heat = mock.MagicMock()
        self.config = ooo_config.Config(heat)
        stack = mock.MagicMock()
        heat.stacks.get.return_value = stack
        stack.outputs = [
            {'output_key': 'RoleNetHostnameMap',
             'output_value': {
                 'Controller': {
                     'ctlplane': [
                         'overcloud-controller-0.ctlplane.localdomain']},
                 'Compute': {
                     'ctlplane': [
                         'overcloud-novacompute-0.ctlplane.localdomain',
                         'overcloud-novacompute-1.ctlplane.localdomain',
                         'overcloud-novacompute-2.ctlplane.localdomain']}}},
            {'output_key': 'ServerIdData',
             'output_value': {
                 'server_ids': {
                     'Controller': [
                         '00b3a5e1-5e8e-4b55-878b-2fa2271f15ad'],
                     'Compute': [
                         'a7db3010-a51f-4ae0-a791-2364d629d20d',
                         '8b07cd31-3083-4b88-a433-955f72039e2c',
                         '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}},
            {'output_key': 'HostnameNetworkConfigMap',
             'output_value': {}},
            {'output_key': 'AnsibleHostVarsMap',
             'output_value': {
                 'Controller': {
                     'overcloud-controller-0': {
                         'uuid': 0,
                         'my_var': 'foo'}},
                 'Compute': {
                     'overcloud-novacompute-0': {
                         'uuid': 1},
                     'overcloud-novacompute-1': {
                         'uuid': 2},
                     'overcloud-novacompute-2': {
                         'uuid': 3}}}},
            {'output_key': 'RoleData',
             'output_value': {
                 'Controller': {
                     'ansible_group_vars': {
                         'chrony_host': 'localhost',
                         'chrony_acl': 'none',
                     }}}},
            {'output_key': 'RoleGroupVars',
             'output_value': {
                 'Controller': {
                     'any_errors_fatal': True,
                     'chrony_host': '192.168.2.1',
                     'chrony_foo': 'bar',
                     'max_fail_percentage': 15},
                 'Compute': {
                     'any_errors_fatal': True,
                     'max_fail_percentage': 15},
                 }}]
        deployment_data, configs = \
            self._get_config_data('config_data.yaml')
        self.configs = configs
        self.deployments = deployment_data
        mock_deployment_data.return_value = deployment_data
        mock_deployment_resource_id.side_effect = self._get_deployment_id
        mock_config_dict.side_effect = self._get_config_dict

        self.tmp_dir = self.useFixture(fixtures.TempDir()).path
        tmp_path = self.config.download_config(stack, self.tmp_dir)

        mock_git_init.assert_called_once_with(self.tmp_dir)
        for f in ['Controller',
                  'Compute', ]:

            with open(os.path.join(tmp_path, 'group_vars', f)) as fin:
                self.assertEqual(
                    self._get_yaml_file(f),
                    yaml.safe_load(fin.read()))

        for f in ['overcloud-controller-0',
                  'overcloud-novacompute-0',
                  'overcloud-novacompute-1',
                  'overcloud-novacompute-2']:
            with open(os.path.join(tmp_path, 'host_vars', f)) as fin:
                self.assertEqual(
                    self._get_yaml_file(os.path.join('host_vars', f)),
                    yaml.safe_load(fin.read()))

        for d in ['ControllerHostEntryDeployment',
                  'NetworkDeployment',
                  'MyExtraConfigPost',
                  'MyPostConfig']:
            with open(os.path.join(tmp_path, 'Controller',
                                   'overcloud-controller-0',
                                   d)) as fin:
                self.assertEqual(
                    yaml.safe_load(fin.read()),
                    self._get_yaml_file(os.path.join(
                                        'overcloud-controller-0',
                                        d)))

        for d in ['ComputeHostEntryDeployment',
                  'NetworkDeployment',
                  'MyExtraConfigPost']:

            with open(os.path.join(tmp_path, 'Compute',
                                   'overcloud-novacompute-0',
                                   d)) as fin:
                self.assertEqual(
                    yaml.safe_load(fin.read()),
                    self._get_yaml_file(os.path.join(
                                        'overcloud-novacompute-0',
                                        d)))

        for d in ['ComputeHostEntryDeployment',
                  'NetworkDeployment',
                  'MyExtraConfigPost']:
            with open(os.path.join(tmp_path, 'Compute',
                                   'overcloud-novacompute-1',
                                   d)) as fin:
                self.assertEqual(
                    yaml.safe_load(fin.read()),
                    self._get_yaml_file(os.path.join(
                                        'overcloud-novacompute-1',
                                        d)))

        for d in ['ComputeHostEntryDeployment',
                  'NetworkDeployment',
                  'MyExtraConfigPost',
                  'AnsibleDeployment']:
            with open(os.path.join(tmp_path, 'Compute',
                                   'overcloud-novacompute-2',
                                   d)) as fin:
                self.assertEqual(
                    yaml.safe_load(fin.read()),
                    self._get_yaml_file(os.path.join(
                                        'overcloud-novacompute-2',
                                        d)))

    @patch.object(ooo_config.Config, 'initialize_git_repo')
    @patch('tripleo_common.utils.config.Config.get_deployment_resource_id')
    @patch('tripleo_common.utils.config.Config.get_config_dict')
    @patch('tripleo_common.utils.config.Config.get_deployment_data')
    def test_config_download_os_apply_config(
        self, mock_deployment_data, mock_config_dict,
        mock_deployment_resource_id, mock_git_init):
        heat = mock.MagicMock()
        self.config = ooo_config.Config(heat)
        stack = mock.MagicMock()
        heat.stacks.get.return_value = stack
        heat.resources.get.return_value = mock.MagicMock()
        stack.outputs = [
            {'output_key': 'RoleNetHostnameMap',
             'output_value': {
                 'Controller': {
                     'ctlplane': [
                         'overcloud-controller-0.ctlplane.localdomain']},
                 'Compute': {
                     'ctlplane': [
                         'overcloud-novacompute-0.ctlplane.localdomain',
                         'overcloud-novacompute-1.ctlplane.localdomain',
                         'overcloud-novacompute-2.ctlplane.localdomain']}}},
            {'output_key': 'ServerIdData',
             'output_value': {
                 'server_ids': {
                     'Controller': [
                         '00b3a5e1-5e8e-4b55-878b-2fa2271f15ad'],
                     'Compute': [
                         'a7db3010-a51f-4ae0-a791-2364d629d20d',
                         '8b07cd31-3083-4b88-a433-955f72039e2c',
                         '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}},
            {'output_key': 'HostnameNetworkConfigMap',
             'output_value': {}},
            {'output_key': 'RoleGroupVars',
             'output_value': {
                 'Controller': {
                     'any_errors_fatal': 'yes',
                     'max_fail_percentage': 15},
                 'Compute': {
                     'any_errors_fatal': 'yes',
                     'max_fail_percentage': 15},
             }}]
        deployment_data, configs = \
            self._get_config_data('config_data.yaml')

        # Add a group:os-apply-config config and deployment
        config_uuid = str(uuid.uuid4())
        configs[config_uuid] = dict(
            id=config_uuid,
            config=dict(a='a'),
            group='os-apply-config',
            outputs=[])

        deployment_uuid = str(uuid.uuid4())
        deployment_mock = mock.MagicMock()
        deployment_mock.id = deployment_uuid
        deployment_mock.attributes = dict(
            value=dict(server='00b3a5e1-5e8e-4b55-878b-2fa2271f15ad',
                       deployment=deployment_uuid,
                       config=config_uuid,
                       name='OsApplyConfigDeployment'))
        deployment_data.append(deployment_mock)

        self.configs = configs
        self.deployments = deployment_data
        mock_deployment_data.return_value = deployment_data
        mock_config_dict.side_effect = self._get_config_dict
        mock_deployment_resource_id.side_effect = self._get_deployment_id

        self.tmp_dir = self.useFixture(fixtures.TempDir()).path
        with warnings.catch_warnings(record=True) as w:
            self.config.download_config(stack, self.tmp_dir)
            mock_git_init.assert_called_once_with(self.tmp_dir)
            # check that we got at least one of the warnings that we expected
            # to throw
            self.assertGreaterEqual(len(w), 1)
            self.assertGreaterEqual(len([x for x in w
                                         if issubclass(x.category,
                                                       DeprecationWarning)]),
                                    1)
            self.assertGreaterEqual(len([x for x in w
                                         if "group:os-apply-config"
                                         in str(x.message)]),
                                    1)

    @patch.object(ooo_config.Config, 'initialize_git_repo')
    @patch('tripleo_common.utils.config.Config.get_deployment_resource_id')
    @patch('tripleo_common.utils.config.Config.get_deployment_data')
    def test_config_download_no_deployment_name(
        self, mock_deployment_data, mock_deployment_resource_id,
        mock_git_init):
        heat = mock.MagicMock()
        self.config = ooo_config.Config(heat)
        stack = mock.MagicMock()
        heat.stacks.get.return_value = stack
        heat.resources.get.return_value = mock.MagicMock()

        deployment_data, _ = self._get_config_data('config_data.yaml')

        # Delete the name of the first deployment and his parent.
        del deployment_data[0].attributes['value']['name']
        deployment_data[0].parent_resource = None
        self.deployments = deployment_data

        mock_deployment_data.return_value = deployment_data
        mock_deployment_resource_id.side_effect = self._get_deployment_id

        self.tmp_dir = self.useFixture(fixtures.TempDir()).path
        self.assertRaises(ValueError,
                          self.config.download_config, stack, self.tmp_dir)
        mock_git_init.assert_called_once_with(self.tmp_dir)

    @patch.object(ooo_config.Config, 'initialize_git_repo')
    @patch('tripleo_common.utils.config.Config.get_deployment_resource_id')
    @patch('tripleo_common.utils.config.Config.get_deployment_data')
    def test_config_download_warn_grandparent_resource_name(
        self, mock_deployment_data, mock_deployment_resource_id,
        mock_git_init):
        heat = mock.MagicMock()
        self.config = ooo_config.Config(heat)
        stack = mock.MagicMock()
        heat.stacks.get.return_value = stack
        heat.resources.get.return_value = mock.MagicMock()

        deployment_data, _ = self._get_config_data('config_data.yaml')

        # Set the name of the deployment to an integer to trigger looking up
        # the grandparent resource name
        deployment_data[0].attributes['value']['name'] = 1
        self.deployments = deployment_data

        mock_deployment_data.return_value = deployment_data
        mock_deployment_resource_id.side_effect = self._get_deployment_id

        self.tmp_dir = self.useFixture(fixtures.TempDir()).path
        with warnings.catch_warnings(record=True) as w:
            self.assertRaises(ValueError,
                              self.config.download_config, stack, self.tmp_dir)
            self.assertGreaterEqual(len(w), 1)
            self.assertGreaterEqual(len([x for x in w
                                         if "grandparent"
                                         in str(x.message)]),
                                    1)

        mock_git_init.assert_called_once_with(self.tmp_dir)

    @patch.object(ooo_config.Config, 'initialize_git_repo')
    @patch('tripleo_common.utils.config.Config.get_deployment_resource_id')
    @patch('tripleo_common.utils.config.Config.get_config_dict')
    @patch('tripleo_common.utils.config.Config.get_deployment_data')
    def test_config_download_no_deployment_uuid(self, mock_deployment_data,
                                                mock_config_dict,
                                                mock_deployment_resource_id,
                                                mock_git_init):
        heat = mock.MagicMock()
        self.config = ooo_config.Config(heat)
        stack = mock.MagicMock()
        heat.stacks.get.return_value = stack
        heat.resources.get.return_value = mock.MagicMock()

        stack.outputs = [
            {'output_key': 'RoleNetHostnameMap',
             'output_value': {
                 'Controller': {
                     'ctlplane': [
                         'overcloud-controller-0.ctlplane.localdomain']},
                 'Compute': {
                     'ctlplane': [
                         'overcloud-novacompute-0.ctlplane.localdomain',
                         'overcloud-novacompute-1.ctlplane.localdomain',
                         'overcloud-novacompute-2.ctlplane.localdomain']}}},
            {'output_key': 'ServerIdData',
             'output_value': {
                 'server_ids': {
                     'Controller': [
                         '00b3a5e1-5e8e-4b55-878b-2fa2271f15ad'],
                     'Compute': [
                         'a7db3010-a51f-4ae0-a791-2364d629d20d',
                         '8b07cd31-3083-4b88-a433-955f72039e2c',
                         '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}},
            {'output_key': 'HostnameNetworkConfigMap',
             'output_value': {}},
            {'output_key': 'RoleGroupVars',
             'output_value': {
                 'Controller': {
                     'any_errors_fatal': 'yes',
                     'max_fail_percentage': 15},
                 'Compute': {
                     'any_errors_fatal': 'yes',
                     'max_fail_percentage': 15},
             }}]
        deployment_data, configs = self._get_config_data('config_data.yaml')

        # Set the deployment to TripleOSoftwareDeployment for the first
        # deployment
        deployment_data[0].attributes['value']['deployment'] = \
            'TripleOSoftwareDeployment'

        # Set the physical_resource_id as '' for the second deployment
        deployment_data[1].attributes['value']['deployment'] = ''

        self.configs = configs
        self.deployments = deployment_data
        mock_deployment_data.return_value = deployment_data
        mock_config_dict.side_effect = self._get_config_dict
        mock_deployment_resource_id.side_effect = self._get_deployment_id

        self.tmp_dir = self.useFixture(fixtures.TempDir()).path
        with warnings.catch_warnings(record=True) as w:
            self.config.download_config(stack, self.tmp_dir)
            assert "Skipping deployment" in str(w[-1].message)
            assert "Skipping deployment" in str(w[-2].message)

    @patch.object(ooo_config.Config, 'initialize_git_repo')
    @patch.object(ooo_config.git, 'Repo')
    @patch.object(ooo_config.shutil, 'copyfile')
    @patch.object(ooo_config.Config, '_mkdir')
    @patch.object(ooo_config.Config, '_open_file')
    @patch.object(ooo_config.shutil, 'rmtree')
    @patch.object(ooo_config.os.path, 'exists')
    def test_overcloud_config_dont_preserve_config(self,
                                                   mock_os_path_exists,
                                                   mock_rmtree,
                                                   mock_open,
                                                   mock_mkdir,
                                                   mock_copyfile,
                                                   mock_repo,
                                                   mock_git_init):
        config_type_list = ['config_settings', 'global_config_settings',
                            'logging_sources', 'monitoring_subscriptions',
                            'service_config_settings',
                            'service_metadata_settings',
                            'service_names',
                            'upgrade_batch_tasks', 'upgrade_tasks',
                            'external_deploy_tasks']

        mock_os_path_exists.get.return_value = True
        heat = mock.MagicMock()
        heat.stacks.get.return_value = fakes.create_tht_stack()
        self.config = ooo_config.Config(heat)
        self.config.fetch_config('overcloud')
        fake_role = list(self.config.stack_outputs.get('RoleData'))
        self.config.download_config('overcloud', '/tmp/tht', config_type_list,
                                    False)

        mock_git_init.assert_called_once_with('/tmp/tht')
        expected_rmtree_calls = [call('/tmp/tht')]
        mock_rmtree.assert_has_calls(expected_rmtree_calls)

        expected_mkdir_calls = [call('/tmp/tht/%s' % r) for r in fake_role]
        mock_mkdir.assert_has_calls(expected_mkdir_calls, any_order=True)
        mock_mkdir.assert_called()
        expected_calls = []
        for config in config_type_list:
            for role in fake_role:
                if 'external' in config:
                    continue
                elif config == 'step_config':
                    expected_calls += [call('/tmp/tht/%s/%s.pp' %
                                            (role, config))]
                elif config == 'param_config':
                    expected_calls += [call('/tmp/tht/%s/%s.json' %
                                            (role, config))]
                else:
                    expected_calls += [call('/tmp/tht/%s/%s.yaml' %
                                            (role, config))]
        mock_open.assert_has_calls(expected_calls, any_order=True)

    @patch.object(ooo_config.shutil, 'rmtree')
    @patch.object(ooo_config.os.path, 'exists')
    def test_create_config_dir(self, mock_os_path_exists, mock_rmtree):
        mock_os_path_exists.get.return_value = True
        heat = mock.MagicMock()
        heat.stacks.get.return_value = fakes.create_tht_stack()
        self.config = ooo_config.Config(heat)
        self.config.create_config_dir('/tmp/tht', False)
        expected_rmtree_calls = [call('/tmp/tht')]
        mock_rmtree.assert_has_calls(expected_rmtree_calls)

    def test_initialize_git_repo(self):
        heat = mock.MagicMock()
        heat.stacks.get.return_value = fakes.create_tht_stack()
        self.config = ooo_config.Config(heat)
        self.tmp_dir = self.useFixture(fixtures.TempDir()).path
        repo = self.config.initialize_git_repo(self.tmp_dir)
        self.assertIsInstance(repo, git.Repo)

    @patch('tripleo_common.utils.config.Config.get_config_dict')
    @patch('tripleo_common.utils.config.Config.get_deployment_data')
    def test_write_config(self, mock_deployment_data, mock_config_dict):
        heat = mock.MagicMock()
        self.config = ooo_config.Config(heat)
        stack = mock.MagicMock()
        heat.stacks.get.return_value = stack

        stack.outputs = [
            {'output_key': 'RoleNetHostnameMap',
             'output_value': {
                 'Controller': {
                     'ctlplane': [
                         'overcloud-controller-0.ctlplane.localdomain']},
                 'Compute': {
                     'ctlplane': [
                         'overcloud-novacompute-0.ctlplane.localdomain',
                         'overcloud-novacompute-1.ctlplane.localdomain',
                         'overcloud-novacompute-2.ctlplane.localdomain']}}},
            {'output_key': 'ServerIdData',
             'output_value': {
                 'server_ids': {
                     'Controller': [
                         '00b3a5e1-5e8e-4b55-878b-2fa2271f15ad'],
                     'Compute': [
                         'a7db3010-a51f-4ae0-a791-2364d629d20d',
                         '8b07cd31-3083-4b88-a433-955f72039e2c',
                         '169b46f8-1965-4d90-a7de-f36fb4a830fe']}}},
            {'output_key': 'RoleData',
             'output_value': {
                 'Controller': {
                     'ansible_group_vars': {
                         'chrony_host': 'localhost',
                         'chrony_acl': 'none',
                     }}}},
            {'output_key': 'RoleGroupVars',
             'output_value': {
                 'Controller': {
                     'any_errors_fatal': True,
                     'chrony_host': '192.168.2.1',
                     'chrony_foo': 'bar',
                     'max_fail_percentage': 15},
                 'Compute': {
                     'any_errors_fatal': True,
                     'max_fail_percentage': 15}}},
            {'output_key': 'HostnameNetworkConfigMap',
             'output_value': {}}
            ]
        deployment_data, configs = \
            self._get_config_data('config_data.yaml')
        self.configs = configs
        self.deployments = deployment_data

        stack_data = self.config.fetch_config('overcloud')
        mock_deployment_data.return_value = deployment_data
        mock_config_dict.side_effect = self._get_config_dict
        config_dir = self.useFixture(fixtures.TempDir()).path

        self.config.write_config(stack_data, 'overcloud', config_dir)

        for f in ['Controller',
                  'Compute', ]:
            with open(os.path.join(config_dir, 'group_vars', f)) as fin:
                self.assertEqual(
                    yaml.safe_load(fin.read()),
                    self._get_yaml_file(f))

        for d in ['ControllerHostEntryDeployment',
                  'NetworkDeployment',
                  'MyExtraConfigPost',
                  'MyPostConfig']:
            with open(os.path.join(config_dir, 'Controller',
                                   'overcloud-controller-0', d)) as fin:
                self.assertEqual(
                    yaml.safe_load(fin.read()),
                    self._get_yaml_file(os.path.join(
                        'overcloud-controller-0',
                        d)))

        for d in ['ComputeHostEntryDeployment',
                  'NetworkDeployment',
                  'MyExtraConfigPost']:
            with open(os.path.join(config_dir, 'Compute',
                                   'overcloud-novacompute-0',
                                   d)) as fin:
                self.assertEqual(
                    yaml.safe_load(fin.read()),
                    self._get_yaml_file(os.path.join(
                        'overcloud-novacompute-0',
                        d)))

        for d in ['ComputeHostEntryDeployment',
                  'NetworkDeployment',
                  'MyExtraConfigPost']:
            with open(os.path.join(config_dir, 'Compute',
                                   'overcloud-novacompute-1',
                                   d)) as fin:
                self.assertEqual(
                    yaml.safe_load(fin.read()),
                    self._get_yaml_file(os.path.join(
                        'overcloud-novacompute-1',
                        d)))

        for d in ['ComputeHostEntryDeployment',
                  'NetworkDeployment',
                  'MyExtraConfigPost',
                  'AnsibleDeployment']:
            with open(os.path.join(config_dir, 'Compute',
                                   'overcloud-novacompute-2', d)) as fin:
                self.assertEqual(
                    yaml.safe_load(fin.read()),
                    self._get_yaml_file(os.path.join(
                        'overcloud-novacompute-2',
                        d)))

    @patch('tripleo_common.utils.config.Config.get_config_dict')
    @patch('tripleo_common.utils.config.Config.get_deployment_data')
    @patch.object(ooo_config.yaml, 'safe_load')
    def test_validate_config(self, mock_yaml, mock_deployment_data,
                             mock_config_dict):
        stack_config = """
        Controller:
          ctlplane:
            overcloud-controller-0.ctlplane.localdomain
        Compute:
          ctlplane:
            overcloud-novacompute-0.ctlplane.localdomain
            overcloud-novacompute-1.ctlplane.localdomain
            overcloud-novacompute-2.ctlplane.localdomain
        """
        yaml_file = '/tmp/testfile.yaml'
        heat = mock.MagicMock()
        heat.stacks.get.return_value = fakes.create_tht_stack()
        self.config = ooo_config.Config(heat)
        self.config.validate_config(stack_config, yaml_file)
        expected_yaml_safe_load_calls = [call(stack_config)]
        mock_yaml.assert_has_calls(expected_yaml_safe_load_calls)

    @patch('tripleo_common.utils.config.Config.get_config_dict')
    @patch('tripleo_common.utils.config.Config.get_deployment_data')
    def test_validate_config_invalid_yaml(self, mock_deployment_data,
                                          mock_config_dict):
        # Use invalid YAML to assert that we properly handle the exception
        stack_config = """
        Controller:
          ctlplane:
            overcloud-controller-0.ctlplane.localdomain
        Compute:
          ctlplane:
        overcloud-novacompute-0.ctlplane.localdomain
        overcloud-novacompute-1.ctlplane.localdomain
        overcloud-novacompute-2.ctlplane.localdomain
        """
        yaml_file = '/tmp/testfile.yaml'
        heat = mock.MagicMock()
        heat.stacks.get.return_value = fakes.create_tht_stack()
        self.config = ooo_config.Config(heat)
        self.assertRaises(yaml.scanner.ScannerError,
                          self.config.validate_config, stack_config, yaml_file)

    @patch('tripleo_common.utils.config.Config.get_network_config_data')
    def test_render_network_config_empty_dict(self,
                                              mock_get_network_config_data):
        heat = mock.MagicMock()
        heat.stacks.get.return_value = fakes.create_tht_stack()
        config_mock = mock.MagicMock()
        config_mock.config = {}
        heat.software_configs.get.return_value = config_mock

        self.config = ooo_config.Config(heat)
        stack = mock.Mock()
        server_roles = dict(Controller='controller')
        mock_get_network_config_data.return_value = dict(Controller='config')
        config_dir = '/tmp/tht'
        self.config.render_network_config(stack, config_dir, server_roles)

    @patch.object(ooo_config.Config, '_open_file')
    @patch('tripleo_common.utils.config.Config.get_network_config_data')
    def test_render_network_config(self,
                                   mock_get_network_config_data,
                                   mock_open):
        heat = mock.MagicMock()
        heat.stacks.get.return_value = fakes.create_tht_stack()
        config_mock = mock.MagicMock()
        config_mock.config = 'some config'
        heat.software_configs.get.return_value = config_mock

        self.config = ooo_config.Config(heat)
        stack = mock.Mock()
        server_roles = dict(node1='Controller')
        mock_get_network_config_data.return_value = dict(node1='config',
                                                         node2='config')
        config_dir = '/tmp/tht'
        self.config.render_network_config(stack, config_dir, server_roles)
        self.assertEqual(2, mock_open.call_count)
        self.assertEqual('/tmp/tht/Controller/node1/NetworkConfig',
                         mock_open.call_args_list[0][0][0])
        self.assertEqual('/tmp/tht/Controller/NetworkConfig',
                         mock_open.call_args_list[1][0][0])


class OvercloudConfigTest(base.TestCase):

    def setUp(self,):
        super(OvercloudConfigTest, self).setUp()
        self.plan = 'overcloud'
        self.delete_after = 3600
        self.config_container = 'config-overcloud'

        # setup swift
        self.template_files = (
            'some-name.yaml',
            'some-other-name.yaml',
            'yet-some-other-name.yaml',
            'finally-another-name.yaml'
        )
        self.swift = mock.MagicMock()
        self.swift.get_container.return_value = (
            {'x-container-meta-usage-tripleo': 'plan'}, [
                {
                    'name': tf,
                    'last_modified': '2018-11-05'
                } for tf in self.template_files
            ]
        )
        self.swift.get_object.return_value = ({}, RESOURCES_YAML_CONTENTS)

    @mock.patch('tripleo_common.utils.swift.delete_container')
    @mock.patch('tripleo_common.utils.swift.download_container')
    @mock.patch('tripleo_common.utils.config.Config.download_config')
    @mock.patch('tripleo_common.utils.tarball.create_tarball')
    def test_get_overcloud_config(self, mock_create_tarball,
                                  mock_config,
                                  mock_swift_download,
                                  mock_swift_delete):
        heat = mock.MagicMock()
        heat.stacks.get.return_value = mock.MagicMock(
            stack_name='stack', id='stack_id')
        mock_config.return_value = '/tmp/fake-path'

        ooo_config.get_overcloud_config(
            self.swift, heat,
            self.plan,
            self.config_container,
            '/tmp/fake-path')

        mock_swift_download.assert_called_once_with(self.swift,
                                                    self.config_container,
                                                    '/tmp/fake-path')
        mock_swift_delete.assert_called_once_with(self.swift,
                                                  self.config_container)
        self.assertEqual(2, self.swift.put_object.call_count)
        self.assertEqual(mock.call(
            'config-overcloud', 'config-overcloud.tar.gz',
            self.swift.put_object.call_args_list[1][0][2]),  # closed file call
            self.swift.put_object.call_args_list[1])
        mock_create_tarball.assert_called_once()
        self.assertEqual(dict(excludes=['.tox', '*.pyc', '*.pyo']),
                         mock_create_tarball.call_args[1])

    @mock.patch('tripleo_common.utils.config.os.unlink')
    @mock.patch('tripleo_common.utils.config.os.path.exists')
    @mock.patch('tripleo_common.utils.config.os.symlink')
    @mock.patch('tripleo_common.utils.swift.download_container')
    @mock.patch('tempfile.mkdtemp')
    def test_download_config(self, mock_mkdtemp,
                             mock_swiftutils,
                             mock_os_symlink,
                             mock_os_path_exists,
                             mock_os_unlink):
        mock_mkdtemp.return_value = '/tmp/tripleo-foo-config'
        ooo_config.download_overcloud_config(self.swift,
                                             self.config_container)
        mock_swiftutils.assert_called_once_with(self.swift,
                                                self.config_container,
                                                '/tmp/tripleo-foo-config')

    @mock.patch('tripleo_common.utils.config.os.path.exists')
    @mock.patch('tripleo_common.utils.config.os.symlink')
    @mock.patch('tripleo_common.utils.swift.download_container')
    @mock.patch('tempfile.mkdtemp')
    def test_download_create_latest_symlink(
            self, mock_mkdtemp,
            mock_swiftutils,
            mock_os_symlink,
            mock_os_path_exists):
        mock_mkdtemp.return_value = '/var/lib/mistral/uuid'
        mock_os_path_exists.return_value = False
        ooo_config.download_overcloud_config(self.swift,
                                             self.config_container)
        mock_swiftutils.assert_called_once_with(self.swift,
                                                self.config_container,
                                                mock_mkdtemp())
        mock_os_symlink.assert_called_once_with(
            '/var/lib/mistral/uuid',
            os.path.join(os.path.dirname('/var/lib/mistral/uuid'),
                         'config-download-latest'))

    @mock.patch('tripleo_common.utils.config.os.unlink')
    @mock.patch('tripleo_common.utils.config.os.path.exists')
    @mock.patch('tripleo_common.utils.config.os.symlink')
    @mock.patch('tripleo_common.utils.swift.download_container')
    @mock.patch('tempfile.mkdtemp')
    def test_download_update_latest_symlink(
            self, mock_mkdtemp,
            mock_swiftutils,
            mock_os_symlink,
            mock_os_path_exists,
            mock_os_unlink):
        mock_mkdtemp.return_value = '/var/lib/mistral/uuid'
        mock_os_path_exists.return_value = True
        ooo_config.download_overcloud_config(self.swift,
                                             self.config_container)
        mock_swiftutils.assert_called_once_with(self.swift,
                                                self.config_container,
                                                mock_mkdtemp())
        mock_os_symlink.assert_called_once_with(
            '/var/lib/mistral/uuid',
            os.path.join(os.path.dirname('/var/lib/mistral/uuid'),
                         'config-download-latest'))
        mock_os_unlink.assert_called_once_with(
            '/var/lib/mistral/config-download-latest')
